@@ -20,7 +20,7 @@
 
 #include "../pureqt.h"
 
-class BootEntry: public QObject {
+class BootEntry final: public QObject {
 public:
 	static QStringList getList();
 private:
@@ -31,7 +31,7 @@ private:
 class BootEntryAction: public U_ACTION {
 	Q_OBJECT
 public:
-	BootEntryAction(const QString &name);
+	explicit BootEntryAction(const QString &name);
 private:
 	Q_DISABLE_COPY(BootEntryAction)
 	QString m_name;
@@ -39,7 +39,7 @@ private:
 
 class BootEntryComboBox: public U_COMBO_BOX {
 public:
-	BootEntryComboBox();
+	explicit BootEntryComboBox();
 private:
 	Q_DISABLE_COPY(BootEntryComboBox)
 };
@@ -47,7 +47,7 @@ private:
 class BootEntryMenu: public U_MENU {
 	Q_OBJECT
 public:
-	BootEntryMenu(QWidget *parent);
+	explicit BootEntryMenu(QWidget *parent);
 private:
 	Q_DISABLE_COPY(BootEntryMenu)
 };
