@@ -19,21 +19,25 @@
 #include "../password.h"
 #include "../pureqt.h"
 
+#include <QCheckBox>
 #include <QDesktopServices>
 #include <QDir>
+#include <QPointer>
+#include <QSettings>
+#include <QUrl>
 
 #ifdef KS_NATIVE_KDE
-	#include <KDesktopFile>
-	#include <KRun>
-	#include <KService>
 	#include <KStandardAction>
-	#include <KStandardDirs>
+	#ifndef KS_KF5
+		#include <KDesktopFile>
+		#include <KRun>
+		#include <KService>
+		#include <KStandardDirs>
+	#endif // KS_KF5
 #endif // KS_NATIVE_KDE
 
 #ifdef KS_PURE_QT
 	#include <QProcess>
-	#include <QSettings>
-	#include <QUrl>
 #endif // KS_PURE_QT
 
 #include "../utils.h"
@@ -66,7 +70,7 @@ bool Extras::onAction() {
 	QFileInfo fileInfo(m_command);
 	QString path = fileInfo.filePath();
 
-#ifdef KS_NATIVE_KDE
+#if defined(KS_NATIVE_KDE) && !defined(KS_KF5)
 	if (KDesktopFile::isDesktopFile(path)) {
 		KDesktopFile desktopFile(m_command);
 		KService service(&desktopFile);
@@ -92,7 +96,7 @@ bool Extras::onAction() {
 		return false;
 	}
 	else {
-		if (KRun::run(m_command, KUrl::List(), U_APP->activeWindow()))
+		if (KRun::run("\"" + m_command + "\"", KUrl::List(), U_APP->activeWindow()))
 			return true;
 		
 		m_error = i18n("Cannot execute \"Extras\" command");
@@ -138,18 +142,29 @@ bool Extras::onAction() {
 #endif // KS_NATIVE_KDE
 }
 
-void Extras::readConfig(const QString &group, Config *config) {
-	config->beginGroup(group);
+void Extras::readConfig(Config *config) {
 	// do not override command set via "e" command line option
 	if (m_command.isNull())
 		setStringOption(config->read("Command", "").toString());
-	config->endGroup();
 }
 
-void Extras::writeConfig(const QString &group, Config *config) {
-	config->beginGroup(group);
+void Extras::updateMainWindow(MainWindow *mainWindow) {
+	if (isEnabled() && getStringOption().isEmpty()) {
+		mainWindow->okCancelButton()->setEnabled(false);
+		mainWindow->infoWidget()->setText(
+			"<qt>" +
+			i18n("Please select an Extras command<br>from the menu above.") +
+			"</qt>",
+			InfoWidget::Type::Warning
+		);
+	}
+	else {
+		Action::updateMainWindow(mainWindow);
+	}
+}
+
+void Extras::writeConfig(Config *config) {
 	config->write("Command", m_command);
-	config->endGroup();
 }
 
 // private
@@ -175,31 +190,34 @@ CommandAction *Extras::createCommandAction(const QFileInfo &fileInfo, const bool
 	QString text = fileInfo.fileName();
 
 	if (!fileInfo.exists() || !fileInfo.isFile())
-		return returnNull ? 0 : new CommandAction(U_STOCK_ICON("dialog-error"), i18n("File not found: %0").arg(text), this, fileInfo.filePath());
-	
-	#ifdef KS_NATIVE_KDE
-	if (KDesktopFile::isDesktopFile(fileInfo.filePath())) {
-		U_ICON icon = readDesktopInfo(fileInfo, text);
+		return returnNull ? 0 : new CommandAction(U_STOCK_ICON("dialog-error"), i18n("File not found: %0").arg(text), this, fileInfo, "");
 
-		return new CommandAction(icon, text, this, fileInfo.filePath());
+	QString statusTip = "";
+
+	if (fileInfo.suffix() == "desktop") {
+		U_ICON icon = readDesktopInfo(fileInfo, text, statusTip);
+
+		return new CommandAction(icon, text, this, fileInfo, statusTip);
 	}
-	#endif // KS_NATIVE_KDE
-	
+
+	statusTip = fileInfo.filePath();
+	statusTip = Utils::trim(statusTip, 100);
+
 	if (fileInfo.isExecutable()) {
 		QString iconName =
 			(fileInfo.suffix() == "sh")
 			? "application-x-executable-script"
 			: "application-x-executable";
-		U_ICON icon = U_STOCK_ICON(iconName);
 
-		return new CommandAction(icon, text, this, fileInfo.filePath());
+		return new CommandAction(U_STOCK_ICON(iconName), text, this, fileInfo, statusTip);
 	}
 
-	return new CommandAction(U_ICON(), text, this, fileInfo.filePath());
+	return new CommandAction(U_ICON(), text, this, fileInfo, statusTip);
 }
 
 U_MENU *Extras::createMenu() {
 	m_menu = new U_MENU();
+	connect(m_menu, SIGNAL(hovered(QAction *)), this, SLOT(onMenuHovered(QAction *)));
 	connect(m_menu, SIGNAL(aboutToShow()), this, SLOT(updateMenu()));
 
 	return m_menu;
@@ -216,17 +234,30 @@ void Extras::createMenu(U_MENU *parentMenu, const QString &parentDir) {
 		fileName = i.fileName();
 		if (i.isDir() && (fileName != ".") && (fileName != "..")) {
 			QString dirProperties = i.filePath() + "/.directory";
-			QString text = i.baseName();
+
 			U_MENU *dirMenu;
 			if (QFile::exists(dirProperties)) {
-				U_ICON icon = readDesktopInfo(dirProperties, text);
-				dirMenu = new U_MENU(text);
+				QString text = i.baseName();
+				QString statusTip = "";
+				U_ICON icon = readDesktopInfo(dirProperties, text, statusTip);
+
+				dirMenu = new U_MENU(text); // use "text" from ".directory" file
 				dirMenu->setIcon(icon);
 			}
 			else {
-				dirMenu = new U_MENU(text);
+				dirMenu = new U_MENU(i.baseName());
 			}
+			connect(dirMenu, SIGNAL(hovered(QAction *)), this, SLOT(onMenuHovered(QAction *)));
+
 			createMenu(dirMenu, i.filePath()); // recursive scan
+
+			if (dirMenu->isEmpty()) {
+				U_ACTION *emptyAction = new U_ACTION(dirMenu);
+				emptyAction->setEnabled(false);
+				emptyAction->setText('(' + i18n("Empty") + ')');
+				dirMenu->addAction(emptyAction);
+			}
+
 			parentMenu->addMenu(dirMenu);
 		}
 		else {
@@ -238,20 +269,20 @@ void Extras::createMenu(U_MENU *parentMenu, const QString &parentDir) {
 }
 
 QString Extras::getFilesDirectory() const {
-#ifdef KS_NATIVE_KDE
+#if defined(KS_NATIVE_KDE) && !defined(KS_KF5)
 	return KGlobal::dirs()->saveLocation("data", "kshutdown/extras");
 #else
-	#ifdef KS_PORTABLE
-	QDir dir = QDir(QApplication::applicationDirPath() + QDir::separator() + "extras");
-	#else
-	
-	#if QT_VERSION >= 0x050000
-	QDir dir = QDir(QStandardPaths::writableLocation(QStandardPaths::DataLocation) + QDir::separator() + "extras");
-	#else
-	QDir dir = QDesktopServices::storageLocation(QDesktopServices::DataLocation) + QDir::separator() + "extras";
-	#endif // QT_VERSION
-	
-	#endif // KS_PORTABLE
+	QDir dir;
+	if (Config::isPortable()) {
+		dir = QDir(QApplication::applicationDirPath() + QDir::separator() + "extras");
+	}
+	else {
+		#if QT_VERSION >= 0x050000
+		dir = QDir(QStandardPaths::writableLocation(QStandardPaths::DataLocation) + QDir::separator() + "extras");
+		#else
+		dir = QDesktopServices::storageLocation(QDesktopServices::DataLocation) + QDir::separator() + "extras";
+		#endif // QT_VERSION
+	}
 
 	//U_DEBUG << "Extras dir: " << dir U_END;
 	// CREDITS: http://stackoverflow.com/questions/6232631/how-to-recursively-create-a-directory-in-qt ;-)
@@ -263,39 +294,50 @@ QString Extras::getFilesDirectory() const {
 #endif // KS_NATIVE_KDE
 }
 
-U_ICON Extras::readDesktopInfo(const QFileInfo &fileInfo, QString &text) {
-#ifdef KS_NATIVE_KDE
+U_ICON Extras::readDesktopInfo(const QFileInfo &fileInfo, QString &text, QString &statusTip) {
+	QString desktopName = "";
+	QString desktopComment = "";
+	QString exec = "";
+	QString icon = "";
+
+#if defined(KS_NATIVE_KDE) && !defined(KS_KF5)
 	KDesktopFile desktopFile(fileInfo.filePath());
+	desktopName = desktopFile.readName();
+	desktopComment = desktopFile.readComment();
+	exec = desktopFile.desktopGroup().readEntry("Exec", "");
+	icon = desktopFile.readIcon();
+#else
+	QSettings settings(fileInfo.filePath(), QSettings::IniFormat);
+	settings.beginGroup("Desktop Entry");
+	desktopName = settings.value("Name", "").toString();
+	desktopComment = settings.value("Comment", "").toString();
+	exec = settings.value("Exec", "").toString();
+	icon = settings.value("Icon", "").toString();
+	settings.endGroup();
+#endif // KS_NATIVE_KDE
+
+	if (!exec.isEmpty())
+		statusTip = Utils::trim(exec, 100);
+
+	if (!desktopComment.isEmpty()) {
+		if (!statusTip.isEmpty())
+			statusTip += '\n';
+		statusTip += Utils::trim(desktopComment, 100);
+	}
 
-	QString desktopName = desktopFile.readName();
 	if (!desktopName.isEmpty())
 		text = desktopName;
 
-	QString desktopComment = desktopFile.readComment();
-	if (!desktopComment.isEmpty())
-		text += (" - " + desktopComment);
+	text = Utils::trim(text, 100);
 
-	QString exec = desktopFile.desktopGroup().readEntry("Exec", "");
-	if (!exec.isEmpty()) {
-		// simplify
-		exec.remove("dbus-send --print-reply --dest=");
-		text += (" - " + exec);
-	}
-
-	return U_STOCK_ICON(desktopFile.readIcon());
-#else
-	Q_UNUSED(fileInfo)//!!!
-	Q_UNUSED(text)
-
-	return U_ICON(); // return dummy icon
-#endif // KS_NATIVE_KDE
+	return U_STOCK_ICON(icon);
 }
 
 void Extras::setCommandAction(const CommandAction *command) {
 	if (command) {
 		m_command = command->m_command;
 		
-		U_DEBUG << "Extras::setCommandAction: " << m_command U_END;
+		//U_DEBUG << "Extras::setCommandAction: " << m_command U_END;
 		m_menuButton->setIcon(U_ICON(command->icon()));
 		m_menuButton->setText(command->text());
 		//m_status = (originalText() + " - " + command->text());
@@ -303,7 +345,7 @@ void Extras::setCommandAction(const CommandAction *command) {
 	else {
 		m_command = QString::null;
 	
-		U_DEBUG << "Extras::setCommandAction: NULL" U_END;
+		//U_DEBUG << "Extras::setCommandAction: NULL" U_END;
 		m_menuButton->setIcon(U_STOCK_ICON("arrow-down"));
 		m_menuButton->setText(i18n("Select a command..."));
 		//m_status = QString::null;
@@ -313,6 +355,10 @@ void Extras::setCommandAction(const CommandAction *command) {
 
 // private slots
 
+void Extras::onMenuHovered(QAction *action) {
+	Utils::showMenuToolTip(action);
+}
+
 void Extras::showHelp() {
 	QDesktopServices::openUrl(QUrl("http://sourceforge.net/p/kshutdown/wiki/Extras/"));
 }
@@ -326,15 +372,45 @@ void Extras::slotModify() {
 		i18n("Use context menu to add/edit/remove actions.") +
 		"<ul>" +
 			"<li>" + i18n("Use <b>Context Menu</b> to create a new link to application (action)") + "</li>" +
+// TODO: built-in "New Folder" menu item
 			"<li>" + i18n("Use <b>Create New|Folder...</b> to create a new submenu") + "</li>" +
 			"<li>" + i18n("Use <b>Properties</b> to change icon, name, or command") + "</li>" +
 		"</ul>" +
 		"</qt>";
 
-	#ifdef KS_NATIVE_KDE
+	#if defined(KS_NATIVE_KDE) && !defined(KS_KF5)
 	KMessageBox::information(0, text, originalText(), "ModifyExtras");
 	#else
-	QMessageBox::information(0, originalText(), text); // krazy:exclude=qclasses
+	Config *config = Config::user();
+	config->beginGroup("KShutdown Action extras");
+	bool showInfo = config->read("Show Info", true).toBool();
+	config->endGroup();
+	
+	if (showInfo) {
+		QPointer<QMessageBox> message = new QMessageBox( // krazy:exclude=qclasses
+			QMessageBox::Information, // krazy:exclude=qclasses
+			originalText(),
+			text,
+			QMessageBox::Ok // krazy:exclude=qclasses
+		);
+
+		#if QT_VERSION >= 0x050200
+		QCheckBox *doNotShowAgainCheckBox = new QCheckBox(i18n("Do not show this message again"));
+		message->setCheckBox(doNotShowAgainCheckBox);
+		#endif // QT_VERSION
+
+		message->exec();
+
+		#if QT_VERSION >= 0x050200
+		if (doNotShowAgainCheckBox->isChecked()) {
+			config->beginGroup("KShutdown Action extras");
+			config->write("Show Info", false);
+			config->endGroup();
+		}
+		#endif // QT_VERSION
+
+		delete message;
+	}
 	#endif // KS_NATIVE_KDE
 	QUrl url = QUrl::fromLocalFile(getFilesDirectory());
 	QDesktopServices::openUrl(url);
@@ -343,14 +419,26 @@ void Extras::slotModify() {
 void Extras::updateMenu() {
 	m_menu->clear();
 
-	#ifdef KS_NATIVE_KDE
+	#if defined(KS_NATIVE_KDE) && !defined(KS_KF5)
 	QStringList dirs(KGlobal::dirs()->findDirs("data", "kshutdown/extras"));
 	foreach (const QString &i, dirs) {
 		U_DEBUG << "Found Extras Directory: " << i U_END;
 		createMenu(m_menu, i);
 	}
 	#else
-	createMenu(m_menu, getFilesDirectory());
+		#ifdef KS_KF5
+		QStringList dirs = QStandardPaths::locateAll(
+			QStandardPaths::GenericDataLocation,
+			"kshutdown/extras",
+			QStandardPaths::LocateDirectory
+		);
+		foreach (const QString &i, dirs) {
+			U_DEBUG << "Found Extras Directory: " << i U_END;
+			createMenu(m_menu, i);
+		}
+		#else
+		createMenu(m_menu, getFilesDirectory());
+		#endif // KS_KF5
 	#endif // KS_NATIVE_KDE
 	
 	if (!m_menu->isEmpty())
@@ -375,9 +463,18 @@ void Extras::updateMenu() {
 
 // private
 
-CommandAction::CommandAction(const U_ICON &icon, QString text, QObject *parent, const QString &command) :
-	U_ACTION(icon, Utils::trim(text, 30), parent),
-	m_command(command) {
+CommandAction::CommandAction(
+	const U_ICON &icon,
+	const QString &text,
+	QObject *parent,
+	const QFileInfo &fileInfo,
+	const QString &statusTip
+) :
+	U_ACTION(icon, text, parent),
+	m_command(fileInfo.filePath()) {
+
+	setIconVisibleInMenu(true);
+	setStatusTip(statusTip);
 	connect(this, SIGNAL(triggered()), SLOT(slotFire()));
 }
 
