@@ -22,7 +22,6 @@
 	#ifndef WIN32_LEAN_AND_MEAN
 		#define WIN32_LEAN_AND_MEAN
 	#endif // WIN32_LEAN_AND_MEAN
-	#define _WIN32_WINNT 0x0500 // for LockWorkStation, etc
 	#include <windows.h>
 #else
 	#include <QDesktopWidget>
@@ -54,10 +53,26 @@ bool LockAction::onAction() {
 
 	return true;
 #else
+// TODO: support custom command in other actions
+	Config *config = Config::user();
+	config->beginGroup("KShutdown Action lock");
+	QString customCommand = config->read("Custom Command", "").toString();
+	config->endGroup();
+
+	customCommand = customCommand.trimmed();
+	if (!customCommand.isEmpty()) {
+		U_DEBUG << "Using custom lock command: " << customCommand U_END;
+		int exitCode = QProcess::execute(customCommand);
+		if (exitCode != 0)
+			U_DEBUG << "Lock command failed to start" U_END;
+		else
+			return true;
+	}
+
 	// HACK: This is a workaround for "lazy" initial kscreensaver repaint.
 	// Now the screen content is hidden immediately.
 	QWidget *blackScreen = 0;
-	if (Utils::isKDE_4()) {
+	if (Utils::isKDE()) {
 		blackScreen = new QWidget(
 			0,
 			Qt::FramelessWindowHint |
@@ -110,7 +125,6 @@ bool LockAction::onAction() {
 			return true;
 	}
 
-// FIXME: GNOME Shell
 	// Unity, GNOME Shell
 	
 	if (Utils::isGNOME() || Utils::isUnity()) {
@@ -145,7 +159,8 @@ bool LockAction::onAction() {
 			return true;
 	}
 
-	// try "lxlock"
+	// LXDE
+
 	if (Utils::isLXDE()) {
 		args.clear();
 		if (launch("lxlock", args))
@@ -162,6 +177,26 @@ bool LockAction::onAction() {
 			return true;
 	}
 
+	// LXQt
+
+	if (Utils::isLXQt()) {
+		args.clear();
+		args << "--lockscreen";
+		if (launch("lxqt-leave", args))
+			return true;
+	}
+
+	// Trinity
+
+	if (Utils::isTrinity()) {
+		args.clear();
+		args << "kdesktop";
+		args << "KScreensaverIface";
+		args << "lock";
+		if (launch("dcop", args))
+			return true;
+	}
+
 	// try "xdg-screensaver" command
 	args.clear();
 	args << "lock";
@@ -189,7 +224,7 @@ QDBusInterface *LockAction::getQDBusInterface() {
 			"/ScreenSaver",
 			"org.freedesktop.ScreenSaver"
 		);
-		if (!m_qdbusInterface->isValid() && Utils::isKDE_4()) {
+		if (!m_qdbusInterface->isValid() && Utils::isKDE()) {
 			delete m_qdbusInterface;
 			
 			U_DEBUG << "LockAction::getQDBusInterface(): using org.kde.krunner" U_END;
