@@ -20,9 +20,14 @@
 #include "password.h"
 #include "utils.h"
 
+#include <QCheckBox>
+#include <QFormLayout>
+#include <QPointer>
+
 // public:
 
 BookmarkAction::BookmarkAction(
+	const QString &text,
 	BookmarksMenu *menu,
 	const QString &actionID, const QString &triggerID,
 	const QString &actionOption, const QString &triggerOption
@@ -42,8 +47,16 @@ BookmarkAction::BookmarkAction(
 	
 	if (action)
 		setIcon(action->icon());
-		
-	setText(menu->makeText(action, trigger, actionOption, triggerOption));
+	setIconVisibleInMenu(true);
+	
+	QString actionText = menu->makeText(action, trigger, actionOption, triggerOption);
+	m_userText = !text.isEmpty() && (text != actionText);
+	m_originalText = m_userText ? text : actionText;
+
+	if (m_userText)
+		setStatusTip(actionText);
+
+	setText(m_originalText);
 }
 
 BookmarkAction::~BookmarkAction() { }
@@ -64,7 +77,8 @@ void BookmarkAction::onAction() {
 	action->setStringOption(m_actionOption);
 	trigger->setStringOption(m_triggerOption);
 	
-	//mainWindow->setActive(true);
+	if (!m_confirmAction)
+		mainWindow->setActive(true);
 }
 
 // public:
@@ -74,6 +88,9 @@ BookmarksMenu::BookmarksMenu(QWidget *parent)
 	m_list(0)
 {
 	connect(this, SIGNAL(aboutToShow()), SLOT(onUpdateMenu()));
+
+	// HACK: workaround for <https://bugreports.qt-project.org/browse/QTBUG-13663>
+	connect(this, SIGNAL(hovered(QAction *)), SLOT(onMenuHovered(QAction *)));
 }
 
 BookmarksMenu::~BookmarksMenu() { }
@@ -89,7 +106,7 @@ QString BookmarksMenu::makeText(KShutdown::Action *action, KShutdown::Trigger *t
 			text += (" - " + Utils::trim(option, 30));
 	}
 	else {
-		text += "?";
+		text += '?';
 	}
 	
 	text += " - ";
@@ -102,7 +119,7 @@ QString BookmarksMenu::makeText(KShutdown::Action *action, KShutdown::Trigger *t
 			text += (" - " + Utils::trim(option, 30));
 	}
 	else {
-		text += "?";
+		text += '?';
 	}
 
 	return text;
@@ -110,8 +127,7 @@ QString BookmarksMenu::makeText(KShutdown::Action *action, KShutdown::Trigger *t
 
 // private:
 
-int BookmarksMenu::findBookmark(KShutdown::Action *action, KShutdown::Trigger *trigger) {
-	int index = 0;
+BookmarkAction *BookmarksMenu::findBookmark(KShutdown::Action *action, KShutdown::Trigger *trigger) {
 	QString actionOption = action->getStringOption();
 	QString triggerOption = trigger->getStringOption();
 	
@@ -120,13 +136,11 @@ int BookmarksMenu::findBookmark(KShutdown::Action *action, KShutdown::Trigger *t
 			(action->id() == i->m_actionID) && (actionOption == i->m_actionOption) &&
 			(trigger->id() == i->m_triggerID) && (triggerOption == i->m_triggerOption)
 		) {
-			return index;
+			return i;
 		}
-		
-		index++;
 	}
 	
-	return -1;
+	return 0;
 }
 
 QList<BookmarkAction *> *BookmarksMenu::list() {
@@ -140,13 +154,17 @@ QList<BookmarkAction *> *BookmarksMenu::list() {
 	int count = config->read("Count", 0).toInt();
 	if (count > 0) {
 		for (int i = 0; i < count; i++) {
-			m_list->append(new BookmarkAction(
+			QString index = QString::number(i);
+			auto *bookmarkAction = new BookmarkAction(
+				config->read("Text " + index, "").toString(),
 				this,
-				config->read("Action " + QString::number(i), "").toString(),
-				config->read("Trigger " + QString::number(i), "").toString(),
-				config->read("Action Option " + QString::number(i), "").toString(),
-				config->read("Trigger Option " + QString::number(i), "").toString()
-			));
+				config->read("Action " + index, "").toString(),
+				config->read("Trigger " + index, "").toString(),
+				config->read("Action Option " + index, "").toString(),
+				config->read("Trigger Option " + index, "").toString()
+			);
+			bookmarkAction->m_confirmAction = config->read("Confirm Action " + index, true).toBool();
+			m_list->append(bookmarkAction);
 		}
 	}
 	config->endGroup();
@@ -161,13 +179,16 @@ void BookmarksMenu::syncConfig() {
 	config->removeAllKeys();
 	config->write("Count", list()->count());
 	
-	int index = 0;
-	foreach (BookmarkAction *i, *list()) {
-		config->write("Action " + QString::number(index), i->m_actionID);
-		config->write("Action Option " + QString::number(index), i->m_actionOption);
-		config->write("Trigger " + QString::number(index), i->m_triggerID);
-		config->write("Trigger Option " + QString::number(index), i->m_triggerOption);
-		index++;
+	int i = 0;
+	foreach (BookmarkAction *bookmarkAction, *list()) {
+		QString index = QString::number(i);
+		config->write("Text " + index, bookmarkAction->m_userText ? bookmarkAction->originalText() : "");
+		config->write("Action " + index, bookmarkAction->m_actionID);
+		config->write("Action Option " + index, bookmarkAction->m_actionOption);
+		config->write("Trigger " + index, bookmarkAction->m_triggerID);
+		config->write("Trigger Option " + index, bookmarkAction->m_triggerOption);
+		config->write("Confirm Action " + index, bookmarkAction->m_confirmAction);
+		i++;
 	}
 	
 	config->endGroup();
@@ -177,9 +198,9 @@ void BookmarksMenu::syncConfig() {
 
 // private slots:
 
-// sort alphabetically, by action text
+// sort alphabetically, by original action text
 bool compareBookmarkAction(const BookmarkAction *a1, const BookmarkAction *a2) {
-	return QString::compare(a1->text(), a2->text(), Qt::CaseInsensitive) < 0;
+	return QString::compare(a1->originalText(), a2->originalText(), Qt::CaseInsensitive) < 0;
 }
 
 void BookmarksMenu::onAddBookmark() {
@@ -190,18 +211,45 @@ void BookmarksMenu::onAddBookmark() {
 	auto *action = mainWindow->getSelectedAction();
 	auto *trigger = mainWindow->getSelectedTrigger();
 
-	BookmarkAction *bookmark = new BookmarkAction(
-		this,
-		action->id(),
-		trigger->id(),
-		action->getStringOption(),
-		trigger->getStringOption()
-	);
-	list()->append(bookmark);
+	QPointer<UDialog> dialog = new UDialog(mainWindow, i18n("Add Bookmark"), false);
+	dialog->acceptButton()->setText(i18n("Add"));
+
+	U_LINE_EDIT *nameField = new U_LINE_EDIT(makeText(action, trigger, QString::null, QString::null));
+	#if QT_VERSION >= 0x050200
+	nameField->setClearButtonEnabled(true);
+	#endif
+
+	auto *confirmActionField = new QCheckBox(i18n("Confirm Action"));
+	confirmActionField->setChecked(true);
+
+	auto *layout = new QFormLayout();
+	layout->setLabelAlignment(Qt::AlignRight);
+	layout->setVerticalSpacing(20_px);
+	layout->addRow(i18n("Name:"), nameField);
+	layout->addRow(confirmActionField);
+	dialog->mainLayout()->addLayout(layout);
+
+	nameField->setFocus();
+	nameField->selectAll();
+
+	if (dialog->exec()) {
+		BookmarkAction *bookmark = new BookmarkAction(
+			nameField->text().trimmed(),
+			this,
+			action->id(),
+			trigger->id(),
+			action->getStringOption(),
+			trigger->getStringOption()
+		);
+		bookmark->m_confirmAction = confirmActionField->isChecked();
+		list()->append(bookmark);
+		
+		qSort(list()->begin(), list()->end(), compareBookmarkAction);
+		
+		syncConfig();
+	}
 	
-	qSort(list()->begin(), list()->end(), compareBookmarkAction);
-	
-	syncConfig();
+	delete dialog;
 }
 
 void BookmarksMenu::onRemoveBookmark() {
@@ -212,32 +260,35 @@ void BookmarksMenu::onRemoveBookmark() {
 	auto *action = mainWindow->getSelectedAction();
 	auto *trigger = mainWindow->getSelectedTrigger();
 
-	int i = findBookmark(action, trigger);
-	if (i != -1) {
-		list()->removeAt(i);
+	auto bookmark = findBookmark(action, trigger);
+	if (bookmark) {
+		list()->removeOne(bookmark);
 		syncConfig();
 	}
 }
 
+void BookmarksMenu::onMenuHovered(QAction *action) {
+	Utils::showMenuToolTip(action);
+}
+
 void BookmarksMenu::onUpdateMenu() {
 	MainWindow *mainWindow = MainWindow::self();
 	auto *action = mainWindow->getSelectedAction();
 	auto *trigger = mainWindow->getSelectedTrigger();
 
-	QString bookmarkName = makeText(action, trigger, QString::null, QString::null);
-
 	U_ACTION *toggleBookmarkAction = new U_ACTION(this);
-	int i = findBookmark(action, trigger);
-	if (i == -1) {
+	auto *bookmark = findBookmark(action, trigger);
+	if (!bookmark) {
 		toggleBookmarkAction->setEnabled(action->canBookmark() && trigger->canBookmark());
 		
 		toggleBookmarkAction->setIcon(U_ICON("bookmark-new"));
-		toggleBookmarkAction->setText(i18n("Add: %0").arg(bookmarkName));
+		QString text = makeText(action, trigger, QString::null, QString::null);
+		toggleBookmarkAction->setText(i18n("Add: %0").arg(text));
 		connect(toggleBookmarkAction, SIGNAL(triggered()), SLOT(onAddBookmark()));
 	}
 	else {
 		toggleBookmarkAction->setIcon(U_ICON("edit-delete"));
-		toggleBookmarkAction->setText(i18n("Remove: %0").arg(bookmarkName));
+		toggleBookmarkAction->setText(i18n("Remove: %0").arg(bookmark->originalText()));
 		connect(toggleBookmarkAction, SIGNAL(triggered()), SLOT(onRemoveBookmark()));
 	}
 
