@@ -24,13 +24,14 @@
 
 #include "kshutdown.h"
 
-class TimeOption {
+class TimeOption final {
 public:
 	inline static KShutdown::Action *action() { return m_action; }
 	inline static void setAction(KShutdown::Action *action) { m_action = action; }
 	static void init();
 	static bool isError();
 	static bool isValid();
+	static QTime parseTime(const QString &time);
 	inline static QString value() { return m_option; }
 	static void setupMainWindow();
 private:
@@ -40,7 +41,7 @@ private:
 	static bool m_relative;
 	static QString m_option;
 	static QTime m_time;
-	TimeOption() { }
+	explicit TimeOption() { }
 };
 
 #endif // KSHUTDOWN_COMMANDLINE_H
