@@ -22,13 +22,12 @@
 	#include <signal.h>
 #endif // KS_UNIX
 
-#include <QDateTimeEdit>
+#include <QPointer>
 #include <QProcess>
 #ifdef Q_OS_WIN32
 	#ifndef WIN32_LEAN_AND_MEAN
 		#define WIN32_LEAN_AND_MEAN
 	#endif // WIN32_LEAN_AND_MEAN
-	#define _WIN32_WINNT 0x0500 // for LockWorkStation, etc
 	#include <windows.h>
 	#include <powrprof.h>
 #else
@@ -40,13 +39,10 @@
 	#include <QDBusReply>
 #endif // KS_DBUS
 
-#ifdef KS_PURE_QT
-	#include <QPointer>
-#endif // KS_PURE_QT
-
 #include "actions/bootentry.h"
 #include "actions/lock.h"
 #include "kshutdown.h"
+#include "log.h"
 #include "mainwindow.h"
 #include "password.h"
 #include "progressbar.h"
@@ -57,10 +53,15 @@ using namespace KShutdown;
 bool Action::m_totalExit = false;
 #ifdef KS_DBUS
 QDBusInterface *Action::m_loginInterface = 0;
+
+QDBusInterface *PowerAction::m_halDeviceInterface = 0;
+QDBusInterface *PowerAction::m_halDeviceSystemPMInterface = 0;
+QDBusInterface *PowerAction::m_upowerInterface = 0;
+
 bool StandardAction::m_kdeShutDownAvailable = false;
 QDBusInterface *StandardAction::m_consoleKitInterface = 0;
-QDBusInterface *StandardAction::m_halInterface = 0;
 QDBusInterface *StandardAction::m_kdeSessionInterface = 0;
+QDBusInterface *StandardAction::m_lxqtSessionInterface = 0;
 QDBusInterface *StandardAction::m_razorSessionInterface = 0;
 #endif // KS_DBUS
 
@@ -69,7 +70,7 @@ QDBusInterface *StandardAction::m_razorSessionInterface = 0;
 // public
 
 Base::Base(const QString &id) :
-	m_statusType(InfoWidget::InfoType),
+	m_statusType(InfoWidget::Type::Info),
 	m_disableReason(QString::null),
 	m_error(QString::null),
 	m_id(id),
@@ -85,8 +86,7 @@ QWidget *Base::getWidget() {
 	return 0;
 }
 
-void Base::readConfig(const QString &group, Config *config) {
-	Q_UNUSED(group)
+void Base::readConfig(Config *config) {
 	Q_UNUSED(config)
 }
 
@@ -94,8 +94,7 @@ void Base::setState(const State state) {
 	Q_UNUSED(state)
 }
 
-void Base::writeConfig(const QString &group, Config *config) {
-	Q_UNUSED(group)
+void Base::writeConfig(Config *config) {
 	Q_UNUSED(config)
 }
 
@@ -105,17 +104,17 @@ void Base::writeConfig(const QString &group, Config *config) {
 // CREDITS: http://lists.trolltech.com/qt-solutions/2005-05/msg00005.html
 void Base::setLastError() {
 	DWORD lastError = ::GetLastError();
-	char *buffer = 0;
-	::FormatMessageA(
+	wchar_t *buffer = 0;
+	::FormatMessageW(
 		FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
 		0,
 		lastError,
 		0,
-		(char *)&buffer,
+		(wchar_t *)&buffer,
 		0,
 		0
 	);
-	m_error = QString::fromLocal8Bit(buffer) + " (error code: " + QString::number(lastError) + ", 0x" + QString::number(lastError, 16) + ')';
+	m_error = QString::fromWCharArray(buffer) + " (error code: " + QString::number(lastError) + ", 0x" + QString::number(lastError, 16) + ')';
 	if (buffer)
 		::LocalFree(buffer);
 }
@@ -133,8 +132,10 @@ Action::Action(const QString &text, const QString &iconName, const QString &id)
 	m_showInMenu(true),
 	m_commandLineArgs(QStringList()) {
 	m_originalText = text;
+
 	if (!iconName.isNull())
 		setIcon(U_STOCK_ICON(iconName));
+	setIconVisibleInMenu(true);
 	
 	if (Utils::isRestricted("kshutdown/action/" + id))
 		disable(i18n("Disabled by Administrator"));
@@ -177,31 +178,47 @@ bool Action::showConfirmationMessage() {
 	else
 		parent = 0;
 
-	#ifdef KS_NATIVE_KDE
-	return KMessageBox::warningYesNo(
-		parent,
-		text,
-		title,
-		KGuiItem(originalText(), KIcon(icon())),
-		KStandardGuiItem::cancel()
-	) == KMessageBox::Yes;
-	#else
-	QPointer<QMessageBox> message = new QMessageBox( // krazy:exclude=qclasses
+	QPointer<QMessageBox> message = new QMessageBox(
 		QMessageBox::Warning,
 		title,
 		text,
-		QMessageBox::Ok | QMessageBox::Cancel, // krazy:exclude=qclasses
+		QMessageBox::Ok | QMessageBox::Cancel,
 		parent
 	);
 	message->setDefaultButton(QMessageBox::Cancel);
 	QAbstractButton *ok = message->button(QMessageBox::Ok);
 	ok->setIcon(icon());
+
+	#ifdef Q_OS_WIN32
+	// HACK: add left/right margins
+	ok->setText(' ' + originalText() + ' ');
+	#else
 	ok->setText(originalText());
+	#endif // Q_OS_WIN32
+
 	bool accepted = message->exec() == QMessageBox::Ok; // krazy:exclude=qclasses
 	delete message;
 	
 	return accepted;
-	#endif // KS_NATIVE_KDE
+}
+
+void Action::updateMainWindow(MainWindow *mainWindow) {
+	if (isEnabled()) {
+		if (mainWindow->active())
+			mainWindow->okCancelButton()->setEnabled(!Utils::isRestricted("kshutdown/action/cancel"));
+		else
+			mainWindow->okCancelButton()->setEnabled(true);
+		emit statusChanged(false);
+	}
+	else {
+		mainWindow->okCancelButton()->setEnabled(false);
+// TODO: show solution dialog
+		QString s = i18n("Action not available: %0").arg(originalText());
+		if (!disableReason().isEmpty())
+			s += ("<br>" + disableReason());
+		mainWindow->infoWidget()->setText("<qt>" + s + "</qt>", InfoWidget::Type::Error);
+	}
+
 }
 
 // protected
@@ -212,6 +229,16 @@ void Action::addCommandLineArg(const QString &shortArg, const QString &longArg)
 		m_commandLineArgs.append(shortArg);
 	if (!longArg.isEmpty())
 		m_commandLineArgs.append(longArg);
+	#ifdef KS_KF5
+	/*
+	QStringList args;
+	if (!shortArg.isEmpty())
+		args << shortArg;
+	if (!longArg.isEmpty())
+		args << longArg;
+	*/
+// TODO: Utils::parser()->addOption(QCommandLineOption(args, originalText()));
+	#endif // KS_KF5
 }
 
 void Action::disable(const QString &reason) {
@@ -271,6 +298,8 @@ bool Action::unsupportedAction() {
 // private slots
 
 void Action::slotFire() {
+	Log::info("Execute action: " + m_id + " (" + m_originalText + ')');
+
 	U_DEBUG << "Action::slotFire() [ id=" << m_id << " ]" U_END;
 
 	if (!isEnabled()) {
@@ -301,9 +330,12 @@ ConfirmAction::ConfirmAction(QObject *parent, Action *action) :
 	// clone basic properties
 	setEnabled(action->isEnabled());
 	setIcon(action->icon());
+	setIconVisibleInMenu(true);
 	setMenu(action->menu());
 	setShortcut(action->shortcut());
+	setStatusTip(action->statusTip());
 	setText(action->text());
+	setToolTip(action->toolTip());
 	connect(this, SIGNAL(triggered()), SLOT(slotFire()));
 }
 
@@ -318,7 +350,7 @@ void ConfirmAction::slotFire() {
 		(m_impl == LockAction::self()) || // lock action - no confirmation
 		m_impl->showConfirmationMessage()
 	) {
-		if (m_impl->authorize(0))
+		if (m_impl->authorize(MainWindow::self()))
 			m_impl->activate(false);
 	}
 }
@@ -332,8 +364,37 @@ Trigger::Trigger(const QString &text, const QString &iconName, const QString &id
 	m_supportsProgressBar(false),
 	m_checkTimeout(500),
 	m_icon(U_STOCK_ICON(iconName)),
-	m_text(text),
-	m_whatsThis(QString::null) {
+	m_text(text) {
+}
+
+// DateTimeEdit
+
+// public:
+
+DateTimeEdit::DateTimeEdit()
+	: QDateTimeEdit()
+{
+	connect(
+		lineEdit(), SIGNAL(selectionChanged()),
+		SLOT(onLineEditSelectionChange())
+	);
+}
+
+DateTimeEdit::~DateTimeEdit() { }
+
+// private slots:
+
+void DateTimeEdit::onLineEditSelectionChange() {
+	if (displayedSections() != (HourSection | MinuteSection))
+		return;
+
+	QString selectedText = lineEdit()->selectedText();
+	// HACK: Change selection from "15h" to "15" after double click.
+	//       This fixes Up/Down keys and mouse wheel editing.
+	if (selectedText == time().toString("hh'h'"))
+		lineEdit()->setSelection(0, 2);
+	else if (selectedText == time().toString("mm'm'"))
+		lineEdit()->setSelection(6, 2);
 }
 
 // DateTimeTriggerBase
@@ -343,8 +404,8 @@ Trigger::Trigger(const QString &text, const QString &iconName, const QString &id
 DateTimeTriggerBase::DateTimeTriggerBase(const QString &text, const QString &iconName, const QString &id) :
 	Trigger(text, iconName, id),
 	m_dateTime(QDateTime()),
-	m_endDateTime(QDateTime()),
-	m_edit(0) {
+	m_endDateTime(QDateTime())
+{
 }
 
 DateTimeTriggerBase::~DateTimeTriggerBase() {
@@ -384,7 +445,7 @@ bool DateTimeTriggerBase::canActivateAction() {
 
 QWidget *DateTimeTriggerBase::getWidget() {
 	if (!m_edit) {
-		m_edit = new QDateTimeEdit();
+		m_edit = new DateTimeEdit();
 		connect(m_edit, SIGNAL(dateChanged(const QDate &)), SLOT(syncDateTime()));
 		connect(m_edit, SIGNAL(timeChanged(const QTime &)), SLOT(syncDateTime()));
 		m_edit->setObjectName("date-time-edit");
@@ -396,19 +457,26 @@ QWidget *DateTimeTriggerBase::getWidget() {
 	return m_edit;
 }
 
-void DateTimeTriggerBase::readConfig(const QString &group, Config *config) {
-	config->beginGroup(group);
-	m_dateTime = config->read("Date Time", m_dateTime).toDateTime();
-	config->endGroup();
+QString DateTimeTriggerBase::longDateTimeFormat() {
+	QString dateFormat = "MMM d dddd";
+	QString timeFormat = QLocale::system()
+		.timeFormat(QLocale::ShortFormat);
+
+	if (timeFormat == "h:mm:ss AP")
+		timeFormat = "h:mm AP";
+	else
+		timeFormat = "hh:mm";
+	
+	return dateFormat + ' ' + timeFormat;
 }
 
-void DateTimeTriggerBase::writeConfig(const QString &group, Config *config) {
-	if (!m_edit)
-		return;
+void DateTimeTriggerBase::readConfig(Config *config) {
+	m_dateTime = config->read("Date Time", m_dateTime).toDateTime();
+}
 
-	config->beginGroup(group);
-	config->write("Date Time", m_edit->dateTime());
-	config->endGroup();
+void DateTimeTriggerBase::writeConfig(Config *config) {
+	if (m_edit)
+		config->write("Date Time", m_edit->dateTime());
 }
 
 void DateTimeTriggerBase::setDateTime(const QDateTime &dateTime) {
@@ -417,7 +485,7 @@ void DateTimeTriggerBase::setDateTime(const QDateTime &dateTime) {
 }
 
 void DateTimeTriggerBase::setState(const State state) {
-	if (state == StartState) {
+	if (state == State::Start) {
 		m_endDateTime = calcEndTime();
 		
 		// reset progress bar
@@ -452,7 +520,7 @@ void DateTimeTriggerBase::syncDateTime() {
 // private
 
 QString DateTimeTriggerBase::createStatus(const QDateTime &now, int &secsTo) {
-	m_statusType = InfoWidget::InfoType;
+	m_statusType = InfoWidget::Type::Info;
 	
 	secsTo = now.secsTo(m_endDateTime);
 	if (secsTo > 0) {
@@ -460,15 +528,19 @@ QString DateTimeTriggerBase::createStatus(const QDateTime &now, int &secsTo) {
 		
 		QString result;
 		if (secsTo < DAY) {
-			result = '+' + QTime(0, 0).addSecs(secsTo).toString(TIME_DISPLAY_FORMAT + "' ':' 'ss's'");
+			result = '+' + QTime(0, 0).addSecs(secsTo).toString(LONG_TIME_DISPLAY_FORMAT);
 		}
 		else {
-			result += "24:00+";
+			result += "24h+";
 		}
 		
 		result += " (";
+
+		if (secsTo >= DAY)
+			result += i18n("not recommended") + ", ";
+
 // TODO: do not bold effective time
-		result += i18n("selected time: %0").arg(m_endDateTime.toString(DATE_TIME_DISPLAY_FORMAT));
+		result += i18n("selected time: %0").arg(m_endDateTime.toString(longDateTimeFormat()));
 		result += ')';
 		
 		return result;
@@ -477,7 +549,7 @@ QString DateTimeTriggerBase::createStatus(const QDateTime &now, int &secsTo) {
 		return QString::null;
 	}
 	else /* if (secsTo < 0) */ {
-		m_statusType = InfoWidget::WarningType;
+		m_statusType = InfoWidget::Type::Warning;
 	
 		return i18n("Invalid date/time");
 	}
@@ -532,10 +604,17 @@ QWidget *DateTimeTrigger::getWidget() {
 	m_edit->setCalendarPopup(true);
 	
 	// Fix for BUG #2444169 - remember the previous shutdown settings
-	if (initDateTime)
-		m_edit->setDateTime(QDateTime(QDate::currentDate(), m_dateTime.time()));
+	if (initDateTime) {
+		QDate date = m_dateTime.date();
+		QTime time = m_dateTime.time();
+
+		// select next day if time is less than current time
+		if ((date == QDate::currentDate()) && (time < QTime::currentTime()))
+			date = date.addDays(1);
+		m_edit->setDateTime(QDateTime(date, time));
+	}
 	
-	m_edit->setDisplayFormat(DATE_TIME_DISPLAY_FORMAT);
+	m_edit->setDisplayFormat(DateTimeTriggerBase::longDateTimeFormat());
 	m_edit->setMinimumDate(QDate::currentDate());
 	//m_edit->setMinimumDateTime(QDateTime::currentDateTime());
 	m_edit->setToolTip(i18n("Enter date and time"));
@@ -546,11 +625,11 @@ QWidget *DateTimeTrigger::getWidget() {
 void DateTimeTrigger::setState(const State state) {
 	DateTimeTriggerBase::setState(state);
 	
-	if (state == InvalidStatusState) {
+	if (state == State::InvalidStatus) {
 		// show warning if selected date/time is invalid
 		if (QDateTime::currentDateTime() >= dateTime()) {
 			m_status = i18n("Invalid date/time");
-			m_statusType = InfoWidget::WarningType;
+			m_statusType = InfoWidget::Type::Warning;
 		}
 		else {
 			updateStatus();
@@ -574,16 +653,6 @@ NoDelayTrigger::NoDelayTrigger() :
 	setCanBookmark(true);
 }
 
-bool NoDelayTrigger::canActivateAction() {
-	return true;
-}
-
-// protected
-
-QDateTime NoDelayTrigger::calcEndTime() {
-	return QDateTime::currentDateTime();
-}
-
 // TimeFromNowTrigger
 
 // public
@@ -642,6 +711,60 @@ PowerAction::PowerAction(const QString &text, const QString &iconName, const QSt
 	setCanBookmark(true);
 }
 
+#ifdef KS_DBUS
+QDBusInterface *PowerAction::getHalDeviceInterface() {
+	if (!m_halDeviceInterface) {
+		// DOC: http://people.freedesktop.org/~dkukawka/hal-spec-git/hal-spec.html
+		m_halDeviceInterface = new QDBusInterface(
+			"org.freedesktop.Hal",
+			"/org/freedesktop/Hal/devices/computer",
+			"org.freedesktop.Hal.Device",
+			QDBusConnection::systemBus()
+		);
+	}
+	if (m_halDeviceInterface->isValid())
+		U_DEBUG << "HAL backend found..." U_END;
+	else
+		U_ERROR << "HAL backend NOT found: " << m_halDeviceInterface->lastError().message() U_END;
+
+	return m_halDeviceInterface;
+}
+
+QDBusInterface *PowerAction::getHalDeviceSystemPMInterface() {
+	if (!m_halDeviceSystemPMInterface) {
+		// DOC: http://people.freedesktop.org/~dkukawka/hal-spec-git/hal-spec.html
+		m_halDeviceSystemPMInterface = new QDBusInterface(
+			"org.freedesktop.Hal",
+			"/org/freedesktop/Hal/devices/computer",
+			"org.freedesktop.Hal.Device.SystemPowerManagement",
+			QDBusConnection::systemBus()
+		);
+	}
+	if (!m_halDeviceSystemPMInterface->isValid())
+		U_ERROR << "No valid org.freedesktop.Hal interface found: " << m_halDeviceSystemPMInterface->lastError().message() U_END;
+
+	return m_halDeviceSystemPMInterface;
+}
+
+QDBusInterface *PowerAction::getUPowerInterface() {
+	// DOC: http://upower.freedesktop.org/docs/UPower.html
+	if (!m_upowerInterface) {
+		m_upowerInterface = new QDBusInterface(
+			"org.freedesktop.UPower",
+			"/org/freedesktop/UPower",
+			"org.freedesktop.UPower",
+			QDBusConnection::systemBus()
+		);
+		if (m_upowerInterface->isValid())
+			U_DEBUG << "UPower backend found..." U_END;
+		else
+			U_DEBUG << "UPower backend NOT found..." U_END;
+	}
+	
+	return m_upowerInterface;
+}
+#endif // KS_DBUS
+
 bool PowerAction::onAction() {
 #ifdef Q_OS_WIN32
 	BOOL hibernate = (m_methodName == "Hibernate");
@@ -661,49 +784,44 @@ bool PowerAction::onAction() {
 		LockAction::self()->activate(false);
 
 		// HACK: wait for screensaver
-		::sleep(2);
+		::sleep(1);
 	}
 	
 	QDBusInterface *login = getLoginInterface();
+	QDBusInterface *upower = getUPowerInterface();
 	
-	// DOC: http://upower.freedesktop.org/docs/UPower.html
-	QDBusInterface i_upower(
-		"org.freedesktop.UPower",
-		"/org/freedesktop/UPower",
-		"org.freedesktop.UPower",
-		QDBusConnection::systemBus()
-	);
-	
-	// DOC: http://people.freedesktop.org/~david/hal-spec/hal-spec.html
-	QDBusInterface i_hal(
-		"org.freedesktop.Hal",
-		"/org/freedesktop/Hal/devices/computer",
-		"org.freedesktop.Hal.Device.SystemPowerManagement",
-		QDBusConnection::systemBus()
-	);
 	
 	QDBusError error;
 	
+	// systemd
+
 	if (login->isValid()) {
-// TODO: test interactivity
+// TODO: test interactivity option
 		login->call(m_methodName, false);
 		
 		error = login->lastError();
 	}
-	else if (i_upower.isValid())
-	{
-		i_upower.call(m_methodName);
+
+	// UPower
+
+	else if (upower->isValid()) {
+		upower->call(m_methodName);
 		
-		error = i_upower.lastError();
+		error = upower->lastError();
 	}
-	else if (i_hal.isValid())
-	{
-		if (m_methodName == "Suspend")
-			i_hal.call(m_methodName, 0);
-		else
-			i_hal.call(m_methodName); // no args
+
+	// HAL (lazy init)
+
+	else {
+		QDBusInterface *hal = getHalDeviceSystemPMInterface();
+		if (hal->isValid()) {
+			if (m_methodName == "Suspend")
+				hal->call(m_methodName, 0);
+			else
+				hal->call(m_methodName); // no args
 		
-		error = i_hal.lastError();
+			error = hal->lastError();
+		}
 	}
 	
 	if (
@@ -726,10 +844,10 @@ bool PowerAction::onAction() {
 
 bool PowerAction::isAvailable(const PowerActionType feature) const {
 #ifdef Q_OS_WIN32
-	if (feature == Hibernate)
+	if (feature == PowerActionType::Hibernate)
 		return ::IsPwrHibernateAllowed();
 
-	if (feature == Suspend)
+	if (feature == PowerActionType::Suspend)
 		return ::IsPwrSuspendAllowed();
 
 	return false;
@@ -741,13 +859,13 @@ bool PowerAction::isAvailable(const PowerActionType feature) const {
 	QDBusInterface *login = getLoginInterface();
 	if (login->isValid()) {
 		switch (feature) {
-			case Suspend: {
+			case PowerActionType::Suspend: {
 				QDBusReply<QString> reply = login->call("CanSuspend");
 				U_DEBUG << "systemd: CanSuspend: " << reply U_END;
 				
 				return reply.isValid() && (reply.value() == "yes");
 			} break;
-			case Hibernate: {
+			case PowerActionType::Hibernate: {
 				QDBusReply<QString> reply = login->call("CanHibernate");
 				U_DEBUG << "systemd: CanHibernate: " << reply U_END;
 				
@@ -756,47 +874,28 @@ bool PowerAction::isAvailable(const PowerActionType feature) const {
 		}
 	}
 
-// TODO: init once
 	// Use the UPower backend if available
-	QDBusInterface i_upower(
-		"org.freedesktop.UPower",
-		"/org/freedesktop/UPower",
-		"org.freedesktop.UPower",
-		QDBusConnection::systemBus()
-	);
-
-// TODO: lazy init
-	// Use the legacy HAL backend if UPower not available
-	QDBusInterface i_hal(
-		"org.freedesktop.Hal",
-		"/org/freedesktop/Hal/devices/computer",
-		"org.freedesktop.Hal.Device",
-		QDBusConnection::systemBus()
-	);
-
-	if (i_upower.isValid())
-	{
-		U_DEBUG << "UPower backend found ..." U_END;
-
+	QDBusInterface *upower = getUPowerInterface();
+	if (upower->isValid()) {
 		switch (feature) {
-			case Suspend:
-				return i_upower.property("CanSuspend").toBool();
-			case Hibernate:
-				return i_upower.property("CanHibernate").toBool();
+			case PowerActionType::Suspend:
+				return upower->property("CanSuspend").toBool();
+			case PowerActionType::Hibernate:
+				return upower->property("CanHibernate").toBool();
 		}
-	}	
-	else if (i_hal.isValid())
-	{
-		U_DEBUG << "HAL backend found ..." U_END;
-			
+	}
+
+	// Use the legacy HAL backend if systemd or UPower not available
+	QDBusInterface *hal = getHalDeviceInterface();
+	if (hal->isValid()) {
 		// try old property name as well, for backward compat.
 		QList<QString> featureNames;
 		switch (feature) {
-			case Suspend:
+			case PowerActionType::Suspend:
 				featureNames.append("power_management.can_suspend");
 				featureNames.append("power_management.can_suspend_to_ram");
 				break;
-			case Hibernate:
+			case PowerActionType::Hibernate:
 				featureNames.append("power_management.can_hibernate");
 				featureNames.append("power_management.can_suspend_to_disk");
 				break;
@@ -806,7 +905,7 @@ bool PowerAction::isAvailable(const PowerActionType feature) const {
 		
 		// Try the alternative feature names in order; if we get a valid answer, return it
 		foreach (const QString &featureName, featureNames) {
-			reply =  i_hal.call("GetProperty", featureName);
+			reply = hal->call("GetProperty", featureName);
 			if (reply.isValid())
 				return reply.value();
 		}
@@ -825,19 +924,12 @@ bool PowerAction::isAvailable(const PowerActionType feature) const {
 HibernateAction::HibernateAction() :
 	PowerAction(i18n("Hibernate Computer"), "system-suspend-hibernate", "hibernate") {
 	m_methodName = "Hibernate";
-	if (!isAvailable(Hibernate))
+	if (!isAvailable(PowerActionType::Hibernate))
 		disable(i18n("Cannot hibernate computer"));
 
 	addCommandLineArg("H", "hibernate");
-	
-	// NOTE: Copied from http://en.wikipedia.org/wiki/Hibernate_(OS_feature)
-	// Do not remove "Source" info.
-	setWhatsThis("<qt>" + i18n(
-"<p>Hibernate (or Suspend-to-Disk) is a feature of many computer operating systems where the contents of RAM are written to non-volatile storage such as a hard disk, as a file or on a separate partition, before powering off the computer.</p>" \
-"<p>When the computer is restarted it reloads the content of memory and is restored to the state it was in when hibernation was invoked.</p>" \
-"<p>Hibernating and later restarting is usually faster than closing down, later starting up, and starting all the programs that were running.</p>" \
-"<p>Source: http://en.wikipedia.org/wiki/Hibernate_(OS_feature)</p>"
-	) + "</qt>");
+
+	setStatusTip(i18n("Save the contents of RAM to disk\nthen turn off the computer."));
 }
 
 // SuspendAction
@@ -853,12 +945,12 @@ SuspendAction::SuspendAction() :
 		#endif // Q_OS_WIN32
 		"system-suspend", "suspend") {
 	m_methodName = "Suspend";
-	if (!isAvailable(Suspend))
+	if (!isAvailable(PowerActionType::Suspend))
 		disable(i18n("Cannot suspend computer"));
 
 	addCommandLineArg("S", "suspend");
-// TODO: menu item tool tips
-	setWhatsThis(i18n("Enter in a low-power state mode."));
+
+	setStatusTip(i18n("Enter in a low-power state mode."));
 }
 
 // StandardAction
@@ -899,6 +991,20 @@ StandardAction::StandardAction(const QString &text, const QString &iconName, con
 			disable("No Razor-qt session found");
 		}
 	}
+
+	if (Utils::isLXQt() && (type == U_SHUTDOWN_TYPE_LOGOUT)) {
+		m_lxqtSessionInterface = new QDBusInterface(
+			"org.lxqt.session",
+			"/LXQtSession",
+			"org.lxqt.session"
+		);
+		QDBusReply<bool> reply = m_lxqtSessionInterface->call("canLogout");
+		if (!reply.isValid() || !reply.value()) {
+			delete m_lxqtSessionInterface;
+			m_lxqtSessionInterface = 0;
+			disable("No LXQt session found");
+		}
+	}
 	#endif // KS_DBUS
 
 	#ifdef KS_UNIX
@@ -1116,7 +1222,27 @@ bool StandardAction::onAction() {
 			#endif // KS_UNIX
 		}
 	}
-	
+
+	// LXQt
+
+	#ifdef KS_DBUS
+	else if (Utils::isLXQt()) {
+		if ((m_type == U_SHUTDOWN_TYPE_LOGOUT) && m_lxqtSessionInterface && m_lxqtSessionInterface->isValid()) {
+			QDBusReply<void> reply = m_lxqtSessionInterface->call("logout");
+
+			if (reply.isValid())
+				return true;
+		}
+	}
+/* DEAD: interactive
+			QStringList args;
+			args << "--logout";
+
+			if (launch("lxqt-leave", args))
+				return true;
+*/
+	#endif // KS_DBUS
+
 	// MATE
 
 	else if (Utils::isMATE()) {
@@ -1128,6 +1254,17 @@ bool StandardAction::onAction() {
 		}
 	}
 
+	// Openbox
+
+	else if (Utils::isOpenbox()) {
+		if (m_type == U_SHUTDOWN_TYPE_LOGOUT) {
+			QStringList args;
+			args << "--exit";
+			if (launch("openbox", args))
+				return true;
+		}
+	}
+
 	// Razor-qt
 	
 	#ifdef KS_DBUS
@@ -1141,6 +1278,25 @@ bool StandardAction::onAction() {
 	}
 	#endif // KS_DBUS
 
+	// Trinity
+
+	else if (Utils::isTrinity()) {
+		if (m_type == U_SHUTDOWN_TYPE_LOGOUT) {
+			QStringList args;
+			args << "ksmserver";
+			args << "ksmserver"; // not a paste bug
+			args << "logout";
+
+			// DOC: http://api.kde.org/4.x-api/kde-workspace-apidocs/plasma-workspace/libkworkspace/html/namespaceKWorkSpace.html
+			args << "0"; // no confirm
+			args << "0"; // logout
+			args << "2"; // force now
+
+			if (launch("dcop", args))
+				return true;
+		}
+	}
+
 	// Xfce
 
 /* FIXME: causes IO error in session manager (?)
@@ -1244,16 +1400,18 @@ bool StandardAction::onAction() {
 			return true;
 	}
 
-	// try HAL
+	// try HAL (lazy init)
+	
+	QDBusInterface *hal = PowerAction::getHalDeviceSystemPMInterface();
 
-	if ((m_type == U_SHUTDOWN_TYPE_HALT) && m_halInterface && m_halInterface->isValid()) {
-		QDBusReply<int> reply = m_halInterface->call("Shutdown");
+	if ((m_type == U_SHUTDOWN_TYPE_HALT) && hal->isValid()) {
+		QDBusReply<int> reply = hal->call("Shutdown");
 
 		if (reply.isValid())
 			return true;
 	}
-	else if ((m_type == U_SHUTDOWN_TYPE_REBOOT) && m_halInterface && m_halInterface->isValid()) {
-		QDBusReply<int> reply = m_halInterface->call("Reboot");
+	else if ((m_type == U_SHUTDOWN_TYPE_REBOOT) && hal->isValid()) {
+		QDBusReply<int> reply = hal->call("Reboot");
 
 		if (reply.isValid())
 			return true;
@@ -1272,15 +1430,19 @@ bool StandardAction::onAction() {
 void StandardAction::checkAvailable(const QString &consoleKitName) {
 	bool available = false;
 	QString error = "";
-
+// TODO: clean up; return bool
 // TODO: win32: check if shutdown/reboot action is available
 
+	// try systemd
+
 	QDBusInterface *login = getLoginInterface();
 	if (login->isValid()) {
 // TODO: CanPowerOff, etc.
 		available = true;
 	}
 
+	// try ConsoleKit
+
 	if (!consoleKitName.isEmpty()) {
 		if (!m_consoleKitInterface) {
 			m_consoleKitInterface = new QDBusInterface(
@@ -1290,6 +1452,17 @@ void StandardAction::checkAvailable(const QString &consoleKitName) {
 				QDBusConnection::systemBus()
 			);
 		}
+		if (!available && !m_consoleKitInterface->isValid()) {
+			// HACK: wait for service start
+			U_DEBUG << "ConsoleKit: Trying again..." U_END;
+			delete m_consoleKitInterface;
+			m_consoleKitInterface = new QDBusInterface(
+				"org.freedesktop.ConsoleKit",
+				"/org/freedesktop/ConsoleKit/Manager",
+				"org.freedesktop.ConsoleKit.Manager",
+				QDBusConnection::systemBus()
+			);
+		}
 		if (m_consoleKitInterface->isValid()) {
 			QDBusReply<bool> reply = m_consoleKitInterface->call(consoleKitName);
 			if (!reply.isValid()) {
@@ -1302,28 +1475,17 @@ void StandardAction::checkAvailable(const QString &consoleKitName) {
 			}
 		}
 		else {
-// FIXME: this sometimes returns error (service timeout?)
-			U_ERROR << "ConsoleKit Error:" << m_consoleKitInterface->lastError().message() U_END;
+			U_ERROR << "ConsoleKit Error: " << m_consoleKitInterface->lastError().message() U_END;
 			if (error.isEmpty())
 				error = "No valid org.freedesktop.ConsoleKit interface found";
 		}
 	}
+	
+	// try HAL (lazy init)
 
-	if (!m_halInterface) {
-		m_halInterface = new QDBusInterface(
-			"org.freedesktop.Hal",
-			"/org/freedesktop/Hal/devices/computer",
-			"org.freedesktop.Hal.Device.SystemPowerManagement",
-			QDBusConnection::systemBus()
-		);
-	}
-	if (m_halInterface->isValid()) {
-		available = true;
-	}
-	else {
-		U_ERROR << "HAL Error:" << m_halInterface->lastError().message() U_END;
-		if (error.isEmpty())
-			error = "No valid org.freedesktop.Hal interface found";
+	if (!available) {
+		QDBusInterface *hal = PowerAction::getHalDeviceSystemPMInterface();
+		available = hal->isValid();
 	}
 	
 	// BUG #19 - disable only if both ConsoleKit and native KDE API is unavailable
@@ -1387,7 +1549,7 @@ RebootAction::RebootAction() :
 	// <http://standards.freedesktop.org/icon-naming-spec/icon-naming-spec-latest.html>
 	setIcon(U_STOCK_ICON("system-reboot"));
 #else
-	if (Utils::isKDE_4())
+	if (Utils::isKDE())
 		setIcon(U_STOCK_ICON("system-reboot"));
 	// HACK: missing "system-reboot" in some icon themes
 	else
