@@ -25,6 +25,7 @@
 #endif // KS_UNIX
 
 #include <QDateTime>
+#include <QDateTimeEdit>
 
 #include "config.h"
 #include "infowidget.h"
@@ -36,20 +37,20 @@ const UShutdownType U_SHUTDOWN_TYPE_REBOOT = 1; // KDE: ShutdownTypeReboot
 const UShutdownType U_SHUTDOWN_TYPE_HALT = 2; // KDE: ShutdownTypeHalt
 
 class BootEntryComboBox;
-class QDateTimeEdit;
+class MainWindow;
 #ifdef KS_DBUS
 class QDBusInterface;
 #endif // KS_DBUS
 
 namespace KShutdown {
 	
-const QString DATE_TIME_DISPLAY_FORMAT = "MMM d dddd hh:mm";
 const QString TIME_DISPLAY_FORMAT = "hh'h ':' 'mm'm'";
+const QString LONG_TIME_DISPLAY_FORMAT = "hh'h ':' 'mm'm ':' 'ss's'";
 const QString TIME_PARSE_FORMAT = "h:mm";
 
-class U_EXPORT Base {
+class Base {
 public:
-	enum State { StartState, StopState, InvalidStatusState };
+	enum class State { Start, Stop, InvalidStatus };
 	explicit Base(const QString &id);
 	virtual ~Base();
 	
@@ -77,7 +78,7 @@ public:
 	inline QString originalText() const {
 		return m_originalText;
 	}
-	virtual void readConfig(const QString &group, Config *config);
+	virtual void readConfig(Config *config);
 	virtual void setState(const State state);
 	inline QString status() const {
 		return m_status;
@@ -85,7 +86,7 @@ public:
 	inline InfoWidget::Type statusType() const {
 		return m_statusType;
 	}
-	virtual void writeConfig(const QString &group, Config *config);
+	virtual void writeConfig(Config *config);
 protected:
 	InfoWidget::Type m_statusType;
 	QString m_disableReason;
@@ -105,7 +106,7 @@ private:
 	bool m_canBookmark;
 };
 
-class U_EXPORT Action: public U_ACTION, public Base {
+class Action: public U_ACTION, public Base {
 	Q_OBJECT
 public:
 	explicit Action(const QString &text, const QString &iconName, const QString &id);
@@ -132,6 +133,7 @@ public:
 	inline static bool totalExit() {
 		return m_totalExit;
 	}
+	virtual void updateMainWindow(MainWindow *mainWindow);
 protected:
 	bool m_force;
 	static bool m_totalExit;
@@ -167,7 +169,7 @@ private slots:
 	void slotFire();
 };
 
-class U_EXPORT Trigger: public QObject, public Base {
+class Trigger: public QObject, public Base {
 	Q_OBJECT
 public:
 	explicit Trigger(const QString &text, const QString &iconName, const QString &id);
@@ -182,8 +184,8 @@ public:
 	inline QString text() const {
 		return m_text;
 	}
-	inline QString whatsThis() const { return m_whatsThis; }
-	inline void setWhatsThis(const QString &value) { m_whatsThis = value; }
+	inline QString toolTip() const { return m_toolTip; }
+	inline void setToolTip(const QString &value) { m_toolTip = value; }
 protected:
 	bool m_supportsProgressBar;
 	int m_checkTimeout;
@@ -191,27 +193,37 @@ private:
 	Q_DISABLE_COPY(Trigger)
 	U_ICON m_icon;
 	QString m_text;
-	QString m_whatsThis;
+	QString m_toolTip = QString::null;
 signals:
 	void notify(const QString &id, const QString &text);
 	void statusChanged(const bool updateWidgets);
 };
 
-class U_EXPORT DateTimeTriggerBase: public Trigger {
+class DateTimeEdit: public QDateTimeEdit {
+	Q_OBJECT
+public:
+	explicit DateTimeEdit();
+	virtual ~DateTimeEdit();
+private slots:
+	void onLineEditSelectionChange();
+};
+
+class DateTimeTriggerBase: public Trigger {
 	Q_OBJECT
 public:
 	explicit DateTimeTriggerBase(const QString &text, const QString &iconName, const QString &id);
 	virtual ~DateTimeTriggerBase();
-	virtual bool canActivateAction();
-	virtual QWidget *getWidget();
-	virtual void readConfig(const QString &group, Config *config);
-	virtual void writeConfig(const QString &group, Config *config);
+	virtual bool canActivateAction() override;
+	virtual QWidget *getWidget() override;
+	static QString longDateTimeFormat();
+	virtual void readConfig(Config *config) override;
+	virtual void writeConfig(Config *config) override;
 	void setDateTime(const QDateTime &dateTime);
-	virtual void setState(const State state);
+	virtual void setState(const State state) override;
 protected:
+	DateTimeEdit *m_edit = nullptr;
 	QDateTime m_dateTime;
 	QDateTime m_endDateTime;
-	QDateTimeEdit *m_edit;
 	virtual QDateTime calcEndTime() = 0;
 	virtual void updateStatus();
 private slots:
@@ -221,77 +233,85 @@ private:
 	QString createStatus(const QDateTime &now, int &secsTo);
 };
 
-class U_EXPORT DateTimeTrigger: public DateTimeTriggerBase {
+class DateTimeTrigger: public DateTimeTriggerBase {
 public:
-	DateTimeTrigger();
+	explicit DateTimeTrigger();
 	QDateTime dateTime();
-	virtual QString getStringOption();
-	virtual void setStringOption(const QString &option);
-	virtual QWidget *getWidget();
-	virtual void setState(const State state);
+	virtual QString getStringOption() override;
+	virtual void setStringOption(const QString &option) override;
+	virtual QWidget *getWidget() override;
+	virtual void setState(const State state) override;
 protected:
-	virtual QDateTime calcEndTime();
+	virtual QDateTime calcEndTime() override;
 private:
 	Q_DISABLE_COPY(DateTimeTrigger)
 };
 
-class U_EXPORT NoDelayTrigger: public Trigger {
+class NoDelayTrigger: public Trigger {
 public:
-	NoDelayTrigger();
-	virtual bool canActivateAction();
-protected:
-	virtual QDateTime calcEndTime();
+	explicit NoDelayTrigger();
+	virtual bool canActivateAction() override { return true; }
 private:
 	Q_DISABLE_COPY(NoDelayTrigger)
 };
 
-class U_EXPORT TimeFromNowTrigger: public DateTimeTriggerBase {
+class TimeFromNowTrigger: public DateTimeTriggerBase {
 public:
-	TimeFromNowTrigger();
-	virtual QString getStringOption();
-	virtual void setStringOption(const QString &option);
-	virtual QWidget *getWidget();
+	explicit TimeFromNowTrigger();
+	virtual QString getStringOption() override;
+	virtual void setStringOption(const QString &option) override;
+	virtual QWidget *getWidget() override;
 protected:
-	virtual QDateTime calcEndTime();
+	virtual QDateTime calcEndTime() override;
 private:
 	Q_DISABLE_COPY(TimeFromNowTrigger)
 };
 
-class U_EXPORT PowerAction: public Action {
+class PowerAction: public Action {
 public:
 	explicit PowerAction(const QString &text, const QString &iconName, const QString &id);
-	virtual bool onAction();
-	enum PowerActionType { Suspend, Hibernate };
+	#ifdef KS_DBUS
+	static QDBusInterface *getHalDeviceInterface();
+	static QDBusInterface *getHalDeviceSystemPMInterface();
+	static QDBusInterface *getUPowerInterface();
+	#endif // KS_DBUS
+	virtual bool onAction() override;
+	enum class PowerActionType { Suspend, Hibernate };
 protected:
 	QString m_methodName;
 	bool isAvailable(const PowerActionType feature) const;
 private:
 	Q_DISABLE_COPY(PowerAction)
+	#ifdef KS_DBUS
+	static QDBusInterface *m_halDeviceInterface;
+	static QDBusInterface *m_halDeviceSystemPMInterface;
+	static QDBusInterface *m_upowerInterface;
+	#endif // KS_DBUS
 };
 
-class U_EXPORT HibernateAction: public PowerAction {
+class HibernateAction: public PowerAction {
 public:
-	HibernateAction();
+	explicit HibernateAction();
 private:
 	Q_DISABLE_COPY(HibernateAction)
 };
 
-class U_EXPORT SuspendAction: public PowerAction {
+class SuspendAction: public PowerAction {
 public:
-	SuspendAction();
+	explicit SuspendAction();
 private:
 	Q_DISABLE_COPY(SuspendAction)
 };
 
-class U_EXPORT StandardAction: public Action {
+class StandardAction: public Action {
 public:
 	explicit StandardAction(const QString &text, const QString &iconName, const QString &id, const UShutdownType type);
-	virtual bool onAction();
+	virtual bool onAction() override;
 protected:
 	#ifdef KS_DBUS
 	static QDBusInterface *m_consoleKitInterface;
-	static QDBusInterface *m_halInterface;
 	static QDBusInterface *m_kdeSessionInterface;
+	static QDBusInterface *m_lxqtSessionInterface;
 	static QDBusInterface *m_razorSessionInterface;
 	void checkAvailable(const QString &consoleKitName);
 	#endif // KS_DBUS
@@ -306,25 +326,25 @@ private:
 	UShutdownType m_type;
 };
 
-class U_EXPORT LogoutAction: public StandardAction {
+class LogoutAction: public StandardAction {
 public:
-	LogoutAction();
+	explicit LogoutAction();
 private:
 	Q_DISABLE_COPY(LogoutAction)
 };
 
-class U_EXPORT RebootAction: public StandardAction {
+class RebootAction: public StandardAction {
 public:
-	RebootAction();
-	virtual QWidget *getWidget();
+	explicit RebootAction();
+	virtual QWidget *getWidget() override;
 private:
 	Q_DISABLE_COPY(RebootAction)
 	BootEntryComboBox *m_bootEntryComboBox;
 };
 
-class U_EXPORT ShutDownAction: public StandardAction {
+class ShutDownAction: public StandardAction {
 public:
-	ShutDownAction();
+	explicit ShutDownAction();
 private:
 	Q_DISABLE_COPY(ShutDownAction)
 };
