@@ -21,27 +21,31 @@
 #include <QCloseEvent>
 #include <QGroupBox>
 #include <QLayout>
+#include <QPointer>
 #include <QTimer>
 
-// TODO: #define KS_CLASSIC
-
 #ifdef KS_DBUS
 	#include <QDBusConnection>
 #endif // KS_DBUS
 
 #ifdef KS_PURE_QT
-	#include <QPointer>
-	#include <QWhatsThis>
-
 	#include "version.h" // for about()
+
+	#include <QLabel>
 #else
 	#include <KActionCollection>
+	#include <KHelpMenu>
 	#include <KNotification>
 	#include <KNotifyConfigWidget>
 	#include <KShortcutsDialog>
 	#include <KStandardAction>
+	#include <KStandardGuiItem>
 #endif // KS_PURE_QT
 
+#ifdef KS_KF5
+	#include <KGlobalAccel>
+#endif // KS_KF5
+
 #include "actions/bootentry.h"
 #include "actions/extras.h"
 #include "actions/lock.h"
@@ -51,10 +55,13 @@
 #include "bookmarks.h"
 #include "commandline.h"
 #include "infowidget.h"
+#include "log.h"
 #include "mainwindow.h"
+#include "mod.h"
 #include "password.h"
 #include "preferences.h"
 #include "progressbar.h"
+#include "stats.h"
 #include "usystemtray.h"
 #include "utils.h"
 
@@ -67,7 +74,7 @@ QList<Trigger*> MainWindow::m_triggerList;
 // public
 
 MainWindow::~MainWindow() {
-	U_DEBUG << "MainWindow::~MainWindow()" U_END;
+	//U_DEBUG << "MainWindow::~MainWindow()" U_END;
 
 	foreach (Action *action, m_actionList)
 		delete action;
@@ -76,6 +83,8 @@ MainWindow::~MainWindow() {
 
 	Config::shutDown();
 	Utils::shutDown();
+	Log::shutDown();
+
 	if (m_progressBar) {
 		delete m_progressBar;
 		m_progressBar = 0;
@@ -83,9 +92,14 @@ MainWindow::~MainWindow() {
 }
 
 bool MainWindow::checkCommandLine() {
-#ifdef KS_PURE_QT
+#if defined(KS_PURE_QT) || defined(KS_KF5)
 	if (Utils::isHelpArg()) {
-		QString table = "<table border=\"1\" cellpadding=\"5\" cellspacing=\"0\" width=\"800\">\n";
+// TODO: get the info directly from QCommandLineParser
+		#ifdef KS_KF5
+		//Utils::parser()->showHelp(0);
+		#endif // KS_KF5
+
+		QString table = "<table border=\"0\" cellpadding=\"2\" cellspacing=\"0\" width=\"800\">\n";
 		
 		table += "<tr><td colspan=\"2\"><b>" + i18n("Actions") + "</b></td></tr>\n";
 		
@@ -94,7 +108,7 @@ bool MainWindow::checkCommandLine() {
 
 			// args
 			
-			table += "<td width=\"50%\"><code>";
+			table += "<td width=\"30%\"><code>";
 			QString argsCell = "";
 			foreach (const QString &arg, action->getCommandLineArgs()) {
 				if (!argsCell.isEmpty())
@@ -118,8 +132,14 @@ bool MainWindow::checkCommandLine() {
 		
 		table += "<tr><td colspan=\"2\"><b>" + i18n("Miscellaneous") + "</b></td></tr>\n";
 
-		if (m_actionHash.contains("idle-monitor"))
-			table += "<tr><td><code>-i, --inactivity</code></td><td>" + i18n("Detect user inactivity. Example: --logout --inactivity 90 - automatically logout after 90 minutes of user inactivity") + "</td></tr>\n";
+		if (m_triggerHash.contains("idle-monitor")) {
+			table += "<tr><td><code>-i, --inactivity</code></td><td>" +
+			i18n( // NOTE: copied from main.cpp
+			"Detect user inactivity. Example:\n"
+			"--logout --inactivity 90 - automatically logout after 90 minutes of user inactivity"
+			).replace('\n', "<br />") +
+			"</td></tr>\n";
+		}
 
 		table += "<tr><td><code>--confirm</code></td><td>" + i18n("Confirm command line action") + "</td></tr>\n";
 
@@ -127,7 +147,22 @@ bool MainWindow::checkCommandLine() {
 		
 		table += "<tr><td><code>--init</code></td><td>" + i18n("Do not show main window on startup") + "</td></tr>\n";
 		
-		table += "<tr><td>" + i18n("Optional parameter") + "</td><td>" + i18n("Activate countdown. Examples: 13:37 - absolute time (HH:MM), 10 - number of minutes from now") + "</td></tr>\n";
+		table += "<tr><td><code>--mod</code></td><td>" + i18n("A list of modifications") + "</td></tr>\n";
+
+		table += "<tr><td><code>--ui-menu</code></td><td>" + i18n("Show custom popup menu instead of main window") + "</td></tr>\n";
+		
+		#ifndef KS_PORTABLE
+		table += "<tr><td><code>--portable</code></td><td>" + i18n("Run in \"portable\" mode") + "</td></tr>\n";
+		#endif // KS_PORTABLE
+		
+		table += "<tr><td>" + i18n("Optional parameter") + "</td><td>" +
+		i18n( // NOTE: copied from main.cpp
+		"Activate countdown. Examples:\n"
+		"13:37 (HH:MM) or \"1:37 PM\" - absolute time\n"
+		"10 or 10m - number of minutes from now\n"
+		"2h - two hours"
+		).replace('\n', "<br />") +
+		"</td></tr>\n";
 
 		// HACK: non-clickable links in Oxygen Style
 		table += "<tr><td colspan=\"2\"><a href=\"http://sourceforge.net/p/kshutdown/wiki/Command%20Line/\">http://sourceforge.net/p/kshutdown/wiki/Command%20Line/</a></td></tr>\n";
@@ -189,16 +224,11 @@ bool MainWindow::checkCommandLine() {
 				
 				return false;
 			}
-			
+
 			// execute action and quit now
-			if (actionToActivate->authorize(0)) {
-				// HACK:
-				if ("test" == actionToActivate->id())
-					actionToActivate->readConfig("KShutdown Action " + actionToActivate->id(), Config::user());
-			
+			if (actionToActivate->authorize(0))
 				actionToActivate->activate(false);
-			}
-			
+
 			return true;
 		}
 	}
@@ -213,14 +243,12 @@ QString MainWindow::getDisplayStatus(const int options) {
 	bool appName = (options & DISPLAY_STATUS_APP_NAME) != 0;
 	bool html = (options & DISPLAY_STATUS_HTML) != 0;
 	bool noAction = (options & DISPLAY_STATUS_HTML_NO_ACTION) != 0;
+	bool okHint = (options & DISPLAY_STATUS_OK_HINT) != 0;
 
 	QString actionText = action->originalText();
 	QString triggerStatus = trigger->status();
 	
 	QString s = "";
-	
-	if (html)
-		s += "<qt>";
 
 	if (html) {
 		if (appName)
@@ -234,7 +262,19 @@ QString MainWindow::getDisplayStatus(const int options) {
 			// HACK: wrap long text
 			int i = s.indexOf(" (");
 			if (i != -1)
-				s.insert(i, "<br>");
+				s.replace(i, 1, "<br>");
+		}
+		
+		if (okHint && m_okCancelButton->isEnabled()) {
+			#ifdef KS_NATIVE_KDE
+			QString okText = KStandardGuiItem::ok().text();
+			okText.remove('&');
+			#else
+			QString okText = i18n("OK");
+			#endif // KS_NATIVE_KDE
+			if (!s.isEmpty())
+				s += "<br><br>";
+			s += i18n("Press %0 to activate KShutdown").arg("<b>" + okText + "</b>");
 		}
 	}
 	// simple - single line, no HTML
@@ -246,7 +286,7 @@ QString MainWindow::getDisplayStatus(const int options) {
 	}
 	
 	if (html)
-		s += "</qt>";
+		s = "<qt>" + s + "</qt>";
 
 	//U_DEBUG << s U_END;
 
@@ -254,7 +294,8 @@ QString MainWindow::getDisplayStatus(const int options) {
 }
 
 void MainWindow::init() {
-	Utils::initArgs();
+	Utils::initArgs(); // 1.
+	Mod::init(); // 2.
 
 	U_DEBUG << "MainWindow::init(): Actions" U_END;
 	m_actionHash = QHash<QString, Action*>();
@@ -283,20 +324,58 @@ void MainWindow::init() {
 		addTrigger(idleMonitor);
 	else
 		delete idleMonitor;
+
+	pluginConfig(true); // read
 }
 
-void MainWindow::maybeShow() {
+bool MainWindow::maybeShow() {
 	if (Utils::isArg("hide-ui")) {
 		hide();
 		m_systemTray->setVisible(false);
 		
-		return;
+		return true;
+	}
+
+	QString menuLayout = Utils::getOption("ui-menu");
+	if (!menuLayout.isEmpty()) {
+		QStringList menuActions = menuLayout.split(':');
+
+		U_MENU *menu = new U_MENU();
+		//connect(menu, SIGNAL(hovered(QAction *)), SLOT(onMenuHovered(QAction *)));
+
+		foreach (const QString &id, menuActions) {
+			if (id == "-") {
+				menu->addSeparator();
+			}
+			else if (id == "cancel") {
+				menu->addAction(m_cancelAction);
+			}
+			else if (id == "quit") {
+				menu->addAction(createQuitAction());
+			}
+			else if (id == "title") {
+				Utils::addTitle(menu, U_APP->windowIcon(), "KShutdown");
+			}
+			else {
+				auto *action = m_actionHash[id];
+// TODO: show confirmation dialog at cursor position (?)
+				if (action)
+					menu->addAction(new ConfirmAction(menu, action));
+				else
+					U_DEBUG << "Unknown ui-menu element: " << id U_END;
+			}
+		}
+
+// FIXME: this does not work in all configurations (e.g. in Qt4 Build only?)
+		menu->exec(QCursor::pos());
+
+		return false;
 	}
 
 	if (!m_systemTray->isSupported()) {
 		show();
 
-		return;
+		return true;
 	}
 
 	bool trayIconEnabled = Config::systemTrayIconEnabled();
@@ -310,6 +389,8 @@ void MainWindow::maybeShow() {
 	else {
 		show();
 	}
+
+	return true;
 }
 
 void MainWindow::setTime(const QString &selectTrigger, const QTime &time, const bool absolute) {
@@ -339,33 +420,35 @@ void MainWindow::setTime(const QString &selectTrigger, const QTime &time, const
 // public slots
 
 QStringList MainWindow::actionList(const bool showDescription) {
-	if (!showDescription)
-		return m_actionHash.keys();
-
 	QStringList sl;
-	foreach (const Action *i, m_actionHash)
-		sl.append(i->id() + " - " + i->originalText());
+	foreach (const Action *i, m_actionList) {
+		if (showDescription)
+			sl.append(i->id() + " - " + i->originalText());
+		else
+			sl.append(i->id());
+	}
 
 	return sl;
 }
 
 QStringList MainWindow::triggerList(const bool showDescription) {
-	if (!showDescription)
-		return m_triggerHash.keys();
-
 	QStringList sl;
-	foreach (const Trigger *i, m_triggerHash)
-		sl.append(i->id() + " - " + i->text());
+	foreach (const Trigger *i, m_triggerList) {
+		if (showDescription)
+			sl.append(i->id() + " - " + i->text());
+		else
+			sl.append(i->id());
+	}
 
 	return sl;
 }
 
 void MainWindow::setActive(const bool yes) {
-	U_DEBUG << "MainWindow::setActive( " << yes << " )" U_END;
-
 	if (m_active == yes)
 		return;
 
+	U_DEBUG << "MainWindow::setActive( " << yes << " )" U_END;
+
 	if (!yes && !PasswordDialog::authorize(this, i18n("Cancel"), "kshutdown/action/cancel"))
 		return;
 
@@ -393,8 +476,8 @@ void MainWindow::setActive(const bool yes) {
 	m_active = yes;
 	m_systemTray->updateIcon(this);
 
-	U_DEBUG << "\tMainWindow::getSelectedAction() == " << action->id() U_END;
-	U_DEBUG << "\tMainWindow::getSelectedTrigger() == " << trigger->id() U_END;
+	//U_DEBUG << "\tMainWindow::getSelectedAction() == " << action->id() U_END;
+	//U_DEBUG << "\tMainWindow::getSelectedTrigger() == " << trigger->id() U_END;
 
 	// reset notifications
 	m_lastNotificationID = QString::null;
@@ -406,8 +489,8 @@ void MainWindow::setActive(const bool yes) {
 			m_confirmLockAction->setEnabled(false);
 #endif // Q_OS_WIN32
 		m_triggerTimer->start(trigger->checkTimeout());
-		action->setState(Base::StartState);
-		trigger->setState(Base::StartState);
+		action->setState(Base::State::Start);
+		trigger->setState(Base::State::Start);
 
 		if (trigger->supportsProgressBar() && Config::user()->progressBarEnabled())
 			m_progressBar->show();
@@ -418,8 +501,8 @@ void MainWindow::setActive(const bool yes) {
 			m_confirmLockAction->setEnabled(true);
 #endif // Q_OS_WIN32
 		m_triggerTimer->stop();
-		action->setState(Base::StopState);
-		trigger->setState(Base::StopState);
+		action->setState(Base::State::Stop);
+		trigger->setState(Base::State::Stop);
 		
 		m_progressBar->hide();
 	}
@@ -443,7 +526,7 @@ void MainWindow::notify(const QString &id, const QString &text) {
 		noHTML.remove("<qt>");
 		noHTML.remove("</qt>");
 	}
-// TODO: show error messages using notification
+
 	KNotification::event(
 		id,
 		noHTML,
@@ -454,10 +537,14 @@ void MainWindow::notify(const QString &id, const QString &text) {
 #endif // KS_NATIVE_KDE
 #ifdef KS_PURE_QT
 	noHTML.replace("<br>", "\n");
-	noHTML.remove(QRegExp("\\<\\w+\\>"));
-	noHTML.remove(QRegExp("\\</\\w+\\>"));
+	noHTML.remove(QRegExp(R"(\<\w+\>)"));
+	noHTML.remove(QRegExp(R"(\</\w+\>)"));
 	m_systemTray->warning(noHTML);
 #endif // KS_PURE_QT
+
+	// flash taskbar button
+	if ((id == "1m") || (id == "5m"))
+		U_APP->alert(this, 10000);
 }
 
 void MainWindow::setExtrasCommand(const QString &command) {
@@ -466,7 +553,7 @@ void MainWindow::setExtrasCommand(const QString &command) {
 }
 
 void MainWindow::setSelectedAction(const QString &id) {
-	U_DEBUG << "MainWindow::setSelectedAction( " << id << " )" U_END;
+	//U_DEBUG << "MainWindow::setSelectedAction( " << id << " )" U_END;
 
 	int index = m_actions->findData(id);
 	if (index == -1)
@@ -478,7 +565,7 @@ void MainWindow::setSelectedAction(const QString &id) {
 }
 
 void MainWindow::setSelectedTrigger(const QString &id) {
-	U_DEBUG << "MainWindow::setSelectedTrigger( " << id << " )" U_END;
+	//U_DEBUG << "MainWindow::setSelectedTrigger( " << id << " )" U_END;
 
 	int index = m_triggers->findData(id);
 	if (index == -1)
@@ -491,13 +578,13 @@ void MainWindow::setTime(const QString &trigger, const QString &time) {
 	if (!trigger.isEmpty())
 		setSelectedTrigger(trigger);
 	
-	QTime t = QTime::fromString(time, KShutdown::TIME_PARSE_FORMAT);
+	QTime t = TimeOption::parseTime(time);
 	bool absolute = (trigger == "date-time") ? true : false;
 	setTime(trigger, t, absolute);
 }
 
 void MainWindow::setWaitForProcess(const qint64 pid) {
-	U_DEBUG << "MainWindow::setWaitForProcess( " << pid << " )" U_END;
+	//U_DEBUG << "MainWindow::setWaitForProcess( " << pid << " )" U_END;
 
 	setActive(false);
 
@@ -559,7 +646,12 @@ void MainWindow::closeEvent(QCloseEvent *e) {
 // private
 
 MainWindow::MainWindow() :
-	U_MAIN_WINDOW(0, Qt::Window),
+	U_MAIN_WINDOW(
+		0,
+		Qt::Window |
+		// disable "Maximize" button; no Qt::WindowMaximizeButtonHint
+		Qt::CustomizeWindowHint | Qt::WindowSystemMenuHint | Qt::WindowTitleHint | Qt::WindowMinimizeButtonHint | Qt::WindowCloseButtonHint
+	),
 	m_active(false),
 	m_forceQuit(false),
 	m_ignoreUpdateWidgets(true),
@@ -571,7 +663,7 @@ MainWindow::MainWindow() :
 	m_currentActionWidget(0),
 	m_currentTriggerWidget(0) {
 
-	U_DEBUG << "MainWindow::MainWindow()" U_END;
+	//U_DEBUG << "MainWindow::MainWindow()" U_END;
 
 #ifdef KS_NATIVE_KDE
 	m_actionCollection = new KActionCollection(this);
@@ -583,9 +675,13 @@ MainWindow::MainWindow() :
 	U_APP->setQuitOnLastWindowClosed(false);
 
 	setObjectName("main-window");
+#ifdef KS_KF5
+// FIXME: ? setWindowIcon(U_ICON(":/images/kshutdown.png"));
+#endif // KF_KF5
 #ifdef KS_PURE_QT
 	// HACK: delete this on quit
 	setAttribute(Qt::WA_DeleteOnClose, true);
+// TODO: wayland <http://blog.martin-graesslin.com/blog/2015/07/porting-qt-applications-to-wayland/>
 	setWindowIcon(U_ICON(":/images/kshutdown.png"));
 #endif // KS_PURE_QT
 
@@ -602,13 +698,12 @@ MainWindow::MainWindow() :
 
 		QString id = action->id();
 		m_actions->addItem(action->icon(), action->text(), id);
-		int index = m_actions->count() - 1;
 		
 		// insert separator like in menu
 		if ((id == "reboot") || (id == "suspend") || (id == "logout"))
 			m_actions->insertSeparator(m_actions->count());
 
-		U_DEBUG << "\tMainWindow::addAction( " << action->text() << " ) [ id=" << id << ", index=" << index << " ]" U_END;
+		//U_DEBUG << "\tMainWindow::addAction( " << action->text() << " ) [ id=" << id << ", index=" << index << " ]" U_END;
 	}
 	m_actions->setMaxVisibleItems(m_actions->count());
 
@@ -625,13 +720,12 @@ MainWindow::MainWindow() :
 
 		QString id = trigger->id();
 		m_triggers->addItem(trigger->icon(), trigger->text(), id);
-		int index = m_triggers->count() - 1;
 		
 		// insert separator
 		if (id == "date-time")
 			m_triggers->insertSeparator(m_triggers->count());
 
-		U_DEBUG << "\tMainWindow::addTrigger( " << trigger->text() << " ) [ id=" << id << ", index=" << index << " ]" U_END;	
+		//U_DEBUG << "\tMainWindow::addTrigger( " << trigger->text() << " ) [ id=" << id << ", index=" << index << " ]" U_END;	
 	}
 	m_triggers->setMaxVisibleItems(m_triggers->count());
 	connect(m_triggerTimer, SIGNAL(timeout()), SLOT(onCheckTrigger()));
@@ -645,14 +739,20 @@ MainWindow::MainWindow() :
 	m_ignoreUpdateWidgets = false;
 	updateWidgets();
 	
+	// HACK: avoid insane window stretching
+	// (dunno how to use setFixedSize correctly w/o breaking layout)
+	QSize hint = sizeHint();
+	setMaximumSize(hint.width() * 2, hint.height() * 2);
+	
 	connect(
 		U_APP, SIGNAL(focusChanged(QWidget *, QWidget *)),
 		this, SLOT(onFocusChange(QWidget *, QWidget *))
 	);
-	
+
 #ifdef KS_DBUS
 	QDBusConnection dbus = QDBusConnection::sessionBus();
 	#ifdef KS_PURE_QT
+// TODO: allow only one application instance
 	dbus.registerService("net.sf.kshutdown");
 	#endif // KS_PURE_QT
 	dbus.registerObject(
@@ -681,30 +781,24 @@ Trigger *MainWindow::getSelectedTrigger() const { // public
 	return m_triggerHash[m_triggers->itemData(m_triggers->currentIndex()).toString()];
 }
 
-void MainWindow::initMenuBar() {
-	U_DEBUG << "MainWindow::initMenuBar()" U_END;
+U_ACTION *MainWindow::createQuitAction() {
+	#ifdef KS_NATIVE_KDE
+	auto *quitAction = KStandardAction::quit(this, SLOT(onQuit()), this);
+	quitAction->setEnabled(!Utils::isRestricted("action/file_quit"));
+	#else
+	auto *quitAction = new U_ACTION(this);
+	quitAction->setIcon(U_STOCK_ICON("application-exit"));
+	quitAction->setShortcut(QKeySequence("Ctrl+Q"));
+	connect(quitAction, SIGNAL(triggered()), SLOT(onQuit()));
+	#endif // KS_NATIVE_KDE
+	// NOTE: Use "Quit KShutdown" instead of "Quit" because
+	// it may be too similar to "Turn Off" in some language translations.
+	quitAction->setText(i18n("Quit KShutdown"));
 
-	U_MENU_BAR *menuBar = new U_MENU_BAR();
-
-	// file menu
-
-	U_MENU *fileMenu = new U_MENU(i18n("A&ction"), menuBar);
-
-	// "No Delay" warning
-	QString warningText = i18n("No Delay");
-#ifdef KS_NATIVE_KDE
-	fileMenu->addTitle(U_STOCK_ICON("dialog-warning"), warningText);
-#else
-	U_ACTION *warningAction = new U_ACTION(menuBar);
-	QFont warningActionFont = warningAction->font();
-	warningActionFont.setBold(true);
-	warningAction->setEnabled(false);
-	warningAction->setFont(warningActionFont);
-	warningAction->setIcon(U_STOCK_ICON("dialog-warning"));
-	warningAction->setText(warningText);
-	fileMenu->addAction(warningAction);
-#endif // KS_NATIVE_KDE
+	return quitAction;
+}
 
+void MainWindow::initFileMenu(U_MENU *fileMenu, const bool addQuitAction) {
 	Action *a;
 	QString id;
 	for (int i = 0; i < m_actions->count(); ++i) {
@@ -727,7 +821,11 @@ void MainWindow::initMenuBar() {
 
 #ifdef KS_NATIVE_KDE
 		m_actionCollection->addAction("kshutdown/" + id, confirmAction);
+		#ifdef KS_KF5
+		KGlobalAccel::setGlobalShortcut(confirmAction, QList<QKeySequence>());
+		#else
 		confirmAction->setGlobalShortcut(KShortcut());
+		#endif // KS_KF5
 // TODO: show global shortcuts: confirmAction->setShortcut(confirmAction->globalShortcut());
 #endif // KS_NATIVE_KDE
 
@@ -750,52 +848,81 @@ void MainWindow::initMenuBar() {
 	fileMenu->addSeparator();
 	fileMenu->addAction(m_cancelAction);
 	//fileMenu->addSeparator();
-#ifdef KS_NATIVE_KDE
-	U_ACTION *quitAction = KStandardAction::quit(this, SLOT(onQuit()), this);
-	quitAction->setEnabled(!Utils::isRestricted("action/file_quit"));
-	fileMenu->addAction(quitAction);
-#else
-	fileMenu->addAction(i18n("Quit"), this, SLOT(onQuit()), QKeySequence("Ctrl+Shift+Q"))
-		->setIcon(U_STOCK_ICON("application-exit"));
-#endif // KS_NATIVE_KDE
+
+	if (addQuitAction)
+		fileMenu->addAction(createQuitAction());
+}
+
+void MainWindow::initMenuBar() {
+	//U_DEBUG << "MainWindow::initMenuBar()" U_END;
+
+	U_MENU_BAR *menuBar = new U_MENU_BAR();
+
+	// file menu
+
+	U_MENU *fileMenu = new U_MENU(i18n("A&ction"), menuBar);
+	connect(fileMenu, SIGNAL(hovered(QAction *)), SLOT(onMenuHovered(QAction *)));
+	Utils::addTitle(fileMenu, /*U_STOCK_ICON("dialog-warning")*/U_ICON(), i18n("No Delay"));
+	initFileMenu(fileMenu, true);
 	menuBar->addMenu(fileMenu);
-	m_systemTray->setContextMenu(fileMenu);
+
+	U_MENU *systemTrayFileMenu = new U_MENU(); // need copy
+// FIXME: wrong tool tip location
+// connect(systemTrayFileMenu, SIGNAL(hovered(QAction *)), SLOT(onMenuHovered(QAction *)));
+	initFileMenu(
+		systemTrayFileMenu,
+		#ifdef KS_KF5
+		!Utils::isKDE()
+		#else
+		true
+		#endif // KS_KF5
+	);
+	m_systemTray->setContextMenu(systemTrayFileMenu);
 
 	// bookmarks menu
-	
-	menuBar->addMenu(m_bookmarksMenu);
 
-	// settings menu
+	if (!Mod::getBool("ui-hide-bookmarks-menu"))
+		menuBar->addMenu(m_bookmarksMenu);
+
+	// tools menu
+
+	U_MENU *toolsMenu = new U_MENU(i18n("&Tools"), menuBar);
+
+	#ifndef Q_OS_WIN32
+	toolsMenu->addAction(i18n("Statistics"), this, SLOT(onStats()));
+	toolsMenu->addSeparator();
+	#endif // Q_OS_WIN32
+
+#ifdef KS_PURE_QT
+	toolsMenu->addAction(i18n("Preferences"), this, SLOT(onPreferences()))
+		->setIcon(U_STOCK_ICON("configure"));
+#endif // KS_PURE_QT
 
-	U_MENU *settingsMenu = new U_MENU(
-		Utils::isGTKStyle() ? i18n("&Edit") : i18n("&Settings"),
-		menuBar
-	);
 #ifdef KS_NATIVE_KDE
 	U_ACTION *configureShortcutsAction = KStandardAction::keyBindings(this, SLOT(onConfigureShortcuts()), this);
 	configureShortcutsAction->setEnabled(!Utils::isRestricted("action/options_configure_keybinding"));
-	settingsMenu->addAction(configureShortcutsAction);
+	toolsMenu->addAction(configureShortcutsAction);
 
 	U_ACTION *configureNotificationsAction = KStandardAction::configureNotifications(this, SLOT(onConfigureNotifications()), this);
 	configureNotificationsAction->setEnabled(!Utils::isRestricted("action/options_configure_notifications"));
-	settingsMenu->addAction(configureNotificationsAction);
+	toolsMenu->addAction(configureNotificationsAction);
 	
-	settingsMenu->addSeparator();
+	toolsMenu->addSeparator();
 	
 	U_ACTION *preferencesAction = KStandardAction::preferences(this, SLOT(onPreferences()), this);
 	preferencesAction->setEnabled(!Utils::isRestricted("action/options_configure"));
-	settingsMenu->addAction(preferencesAction);
-#else
-	settingsMenu->addAction(i18n("Preferences"), this, SLOT(onPreferences()))
-		->setIcon(U_STOCK_ICON("configure"));
+	toolsMenu->addAction(preferencesAction);
 #endif // KS_NATIVE_KDE
-	menuBar->addMenu(settingsMenu);
+
+	menuBar->addMenu(toolsMenu);
 
 	// help menu
 
 #ifdef KS_NATIVE_KDE
 	Config *config = Config::user();
 	config->beginGroup("KDE Action Restrictions");
+	// unused
+	config->write("action/help_whats_this", false);
 	// KDE version is in About dialog too
 	config->write("action/help_about_kde", false);
 	// no KShutdown Handbook yet
@@ -803,17 +930,28 @@ void MainWindow::initMenuBar() {
 	// mail bug report does not work (known bug)
 	config->write("action/help_report_bug", false);
 	config->endGroup();
+	#ifdef KS_KF5
+	KHelpMenu *helpMenu = new KHelpMenu(this);
+	menuBar->addMenu(helpMenu->menu());
+	#else
 	menuBar->addMenu(helpMenu());
+	#endif // KS_KF5
 #else
 	U_MENU *helpMenu = new U_MENU(i18n("&Help"), menuBar);
-	helpMenu->addAction(QWhatsThis::createAction(this));
-	helpMenu->addSeparator();
 	helpMenu->addAction(i18n("About"), this, SLOT(onAbout()));
-	helpMenu->addAction(i18n("About Qt"), U_APP, SLOT(aboutQt()));
 	menuBar->addMenu(helpMenu);
 #endif // KS_NATIVE_KDE
 
 	setMenuBar(menuBar);
+
+	if (Mod::getBool("ui-hide-menu-bar"))
+		menuBar->hide();
+	// HACK: Fixes Bookmarks menu and key shortcuts
+	//       (Global app menus in Unity are all f*** - it's a fact)
+	// BUG: https://bugs.launchpad.net/appmenu-qt5/+bug/1449373
+	//      https://bugs.launchpad.net/appmenu-qt5/+bug/1380702
+	else
+		menuBar->show();
 }
 
 void MainWindow::initWidgets() {
@@ -824,45 +962,59 @@ void MainWindow::initWidgets() {
 	QWidget *mainWidget = new QWidget();
 	mainWidget->setObjectName("main-widget");
 	QVBoxLayout *mainLayout = new QVBoxLayout(mainWidget);
-	mainLayout->setMargin(5);
-	mainLayout->setSpacing(10);
+	mainLayout->setMargin(5_px);
+	mainLayout->setSpacing(10_px);
 
-	m_actionBox = new QGroupBox(i18n("Selec&t an action"));
+	m_actionBox = new QGroupBox(i18n("Select an &action"));
 	m_actionBox->setObjectName("action-box");
-	m_actionBox->setLayout(new QVBoxLayout());
+
+	auto *actionLayout = new QVBoxLayout(m_actionBox);
+	actionLayout->setMargin(5_px);
+	actionLayout->setSpacing(10_px);
 
 	m_actions = new U_COMBO_BOX();
 	m_actions->setObjectName("actions");
 	connect(m_actions, SIGNAL(activated(int)), SLOT(onActionActivated(int)));
-	m_actionBox->layout()->addWidget(m_actions);
+	actionLayout->addWidget(m_actions);
 
 	m_force = new QCheckBox(i18n("Do not save session / Force shutdown"));
 	m_force->setObjectName("force");
 	connect(m_force, SIGNAL(clicked()), SLOT(onForceClick()));
-	m_actionBox->layout()->addWidget(m_force);
+	actionLayout->addWidget(m_force);
 
 	m_triggerBox = new QGroupBox(i18n("Se&lect a time/event"));
 	m_triggerBox->setObjectName("trigger-box");
-	m_triggerBox->setLayout(new QVBoxLayout());
+
+	auto *triggerLayout = new QVBoxLayout(m_triggerBox);
+	triggerLayout->setMargin(5_px);
+	triggerLayout->setSpacing(10_px);
 
 	m_triggers = new U_COMBO_BOX();
 	m_triggers->setObjectName("triggers");
 	connect(m_triggers, SIGNAL(activated(int)), SLOT(onTriggerActivated(int)));
-	m_triggerBox->layout()->addWidget(m_triggers);
+	triggerLayout->addWidget(m_triggers);
 	
 	m_infoWidget = new InfoWidget(this);
 
 	m_okCancelButton = new U_PUSH_BUTTON();
 	m_okCancelButton->setObjectName("ok-cancel-button");
+	
 	m_okCancelButton->setDefault(true);
 	connect(m_okCancelButton, SIGNAL(clicked()), SLOT(onOKCancel()));
 
+	// add extra margin around OK/Cancel button
+	//m_okCancelButton->setContentsMargins(10_px, 10_px, 10_px, 10_px); NOP
+	auto *okCancelWrapper = new QWidget(mainWidget);
+	auto *okCancelLayout = new QVBoxLayout(okCancelWrapper);
+	okCancelLayout->setMargin(5_px);
+	okCancelLayout->addWidget(m_okCancelButton);
+
 	mainLayout->addWidget(m_actionBox);
 	mainLayout->addWidget(m_triggerBox);
 	mainLayout->addStretch();
 	mainLayout->addWidget(m_infoWidget);
 	mainLayout->addStretch();
-	mainLayout->addWidget(m_okCancelButton);
+	mainLayout->addWidget(okCancelWrapper);
 
 	setCentralWidget(mainWidget);
 	
@@ -870,46 +1022,42 @@ void MainWindow::initWidgets() {
 #ifdef KS_NATIVE_KDE
 	m_cancelAction->setIcon(KStandardGuiItem::cancel().icon());
 	m_actionCollection->addAction("kshutdown/cancel", m_cancelAction);
+	#ifdef KS_KF5
+	KGlobalAccel::setGlobalShortcut(m_cancelAction, QList<QKeySequence>());
+	#else
 	m_cancelAction->setGlobalShortcut(KShortcut());
+	#endif // KS_KF5
 // TODO: show global shortcut: m_cancelAction->setShortcut(m_cancelAction->globalShortcut());
 #else
 	m_cancelAction->setIcon(U_STOCK_ICON("dialog-cancel"));
 #endif // KS_NATIVE_KDE
 	connect(m_cancelAction, SIGNAL(triggered()), SLOT(onCancel()));
-
-	#ifdef KS_CLASSIC
-	QPalette windowPalette;
-	windowPalette.setColor(QPalette::Window, QColor(0xA7DC6B));
-	windowPalette.setColor(QPalette::WindowText, Qt::black);
-	setPalette(windowPalette);
-	#endif // KS_CLASSIC
 }
 
 void MainWindow::pluginConfig(const bool read) {
 	Config *config = Config::user();
-	QString group;
 
 	foreach (Action *i, m_actionList) {
-		group = "KShutdown Action " + i->id();
+		config->beginGroup("KShutdown Action " + i->id());
 		if (read)
-			i->readConfig(group, config);
+			i->readConfig(config);
 		else
-			i->writeConfig(group, config);
+			i->writeConfig(config);
+		config->endGroup();
 	}
 
 	foreach (Trigger *i, m_triggerList) {
-		group = "KShutdown Trigger " + i->id();
+		config->beginGroup("KShutdown Trigger " + i->id());
 		if (read)
-			i->readConfig(group, config);
+			i->readConfig(config);
 		else
-			i->writeConfig(group, config);
+			i->writeConfig(config);
+		config->endGroup();
 	}
 }
 
 void MainWindow::readConfig() {
-	U_DEBUG << "MainWindow::readConfig()" U_END;
-
-	pluginConfig(true); // read
+	//U_DEBUG << "MainWindow::readConfig()" U_END;
 
 	Config *config = Config::user();
 	config->beginGroup("General");
@@ -923,7 +1071,9 @@ void MainWindow::readConfig() {
 }
 
 void MainWindow::setTitle(const QString &plain, const QString &html) {
-#ifdef KS_NATIVE_KDE
+#ifdef KS_KF5
+	setWindowTitle(plain);
+#elif defined(KS_NATIVE_KDE)
 	setCaption(plain);
 #else
 	if (plain.isEmpty())
@@ -939,6 +1089,10 @@ void MainWindow::setTitle(const QString &plain, const QString &html) {
 	m_systemTray->setToolTip(s);
 	#endif // Q_OS_WIN32
 
+	// HACK: add "PB" prefix
+	if (s.startsWith("<qt>"))
+		s.insert(4, i18n("Progress Bar") + " - ");
+
 	m_progressBar->setToolTip(s);
 }
 
@@ -962,27 +1116,28 @@ void MainWindow::updateWidgets() {
 	m_bookmarksMenu->setEnabled(enabled);
 	m_force->setEnabled(enabled);
 
-	bool canCancel = !Utils::isRestricted("kshutdown/action/cancel");
-
-	#ifdef KS_CLASSIC
-	QPalette buttonPalette;
-	if (m_active) {
-		buttonPalette.setColor(QPalette::Button, QColor(0xDCD86A));
-		buttonPalette.setColor(QPalette::ButtonText, Qt::black);
-	}
-	else {
-		buttonPalette.setColor(QPalette::Button, QColor(0xDC6A6E));
-		buttonPalette.setColor(QPalette::ButtonText, Qt::white);
-	}
-	m_okCancelButton->setPalette(buttonPalette);
-	#endif // KS_CLASSIC
+	Mod::applyMainWindowColors(this);
 
 #ifdef KS_NATIVE_KDE
+	#ifdef KS_KF5
+	bool hasIcon = m_okCancelButton->style()->styleHint(QStyle::SH_DialogButtonBox_ButtonsHaveIcons);
+	if (m_active) {
+		if (hasIcon)
+			m_okCancelButton->setIcon(U_STOCK_ICON("dialog-cancel"));
+		m_okCancelButton->setText(i18n("Cancel"));
+	}
+	else {
+		if (hasIcon)
+			m_okCancelButton->setIcon(U_STOCK_ICON("dialog-ok"));
+		m_okCancelButton->setText(i18n("OK"));
+	}
+	#else
 	m_okCancelButton->setGuiItem(
 		m_active
 		? KStandardGuiItem::cancel()
 		: KStandardGuiItem::ok()
 	);
+	#endif // KS_KF5
 #else
 	bool hasIcon = m_okCancelButton->style()->styleHint(QStyle::SH_DialogButtonBox_ButtonsHaveIcons);
 	if (m_active) {
@@ -999,37 +1154,11 @@ void MainWindow::updateWidgets() {
 	m_okCancelButton->setToolTip(i18n("Click to activate/cancel the selected action"));
 
 	Action *action = getSelectedAction();
-	if (action->isEnabled()) {
-		if ((action == Extras::self()) && Extras::self()->getStringOption().isEmpty()) {
-			m_okCancelButton->setEnabled(false);
-// TODO: move to Extras class
-			m_infoWidget->setText(
-				"<qt>" +
-				i18n("Please select an Extras command<br>from the menu above.") +
-				"</qt>",
-				InfoWidget::WarningType
-			);
-		}
-		else {
-			if (m_active)
-				m_okCancelButton->setEnabled(canCancel);
-			else
-				m_okCancelButton->setEnabled(true);
-			onStatusChange(false);
-		}
-	}
-	else {
-		m_okCancelButton->setEnabled(false);
-// TODO: show solution dialog
-		QString s = i18n("Action not available: %0").arg(action->originalText());
-		if (!action->disableReason().isEmpty())
-			s += ("<br>" + action->disableReason());
-		m_infoWidget->setText("<qt>" + s + "</qt>", InfoWidget::ErrorType);
-	}
-	
+	action->updateMainWindow(this);
+
 	// update "Cancel" action
 	if (m_active) {
-		m_cancelAction->setEnabled(canCancel);
+		m_cancelAction->setEnabled(!Utils::isRestricted("kshutdown/action/cancel"));
 		m_cancelAction->setText(i18n("Cancel: %0").arg(action->originalText()));
 	}
 	else {
@@ -1039,7 +1168,7 @@ void MainWindow::updateWidgets() {
 }
 
 void MainWindow::writeConfig() { // public
-	U_DEBUG << "MainWindow::writeConfig()" U_END;
+	//U_DEBUG << "MainWindow::writeConfig()" U_END;
 
 	pluginConfig(false); // write
 
@@ -1059,9 +1188,9 @@ void MainWindow::writeConfig() { // public
 // public slots
 
 void MainWindow::onQuit() {
-	U_DEBUG << "MainWindow::onQuit()" U_END;
+	//U_DEBUG << "MainWindow::onQuit()" U_END;
 	
-	if (!PasswordDialog::authorize(this, i18n("Quit"), "action/file_quit"))
+	if (!PasswordDialog::authorize(this, i18n("Quit KShutdown"), "action/file_quit"))
 		return;
 
 	m_forceQuit = true;
@@ -1074,34 +1203,95 @@ void MainWindow::onQuit() {
 
 #ifdef KS_PURE_QT
 void MainWindow::onAbout() {
-	QString version = KS_FULL_VERSION;
-#ifdef KS_PORTABLE
-	version += " (portable)";
-#endif // KS_PORTABLE
+	auto *iconLabel = new QLabel();
+	iconLabel->setAlignment(Qt::AlignCenter);
+	iconLabel->setPixmap(U_ICON(":/images/hi64-app-kshutdown.png").pixmap(64, 64));
+	// TEST: iconLabel->setPixmap(U_ICON(":/images/hi128-app-kshutdown.png").pixmap(128, 128));
 
-QString license =
-"This program is <b>free software</b>; you can redistribute it and/or modify\n" \
-"it under the terms of the GNU General Public License as published by\n" \
-"the Free Software Foundation; either version 2 of the License, or\n" \
-"(at your option) any later version.\n" \
-"\n" \
-"This program is distributed in the hope that it will be useful,\n" \
-"but <b>WITHOUT ANY WARRANTY</b>; without even the implied warranty of\n" \
-"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n" \
-"GNU General Public License for more details.\n" \
-"&lt;<a href=\"http://www.gnu.org/licenses/\">http://www.gnu.org/licenses/</a>&gt;";
-
-	QMessageBox::about( // krazy:exclude=qclasses
-		this,
-		i18n("About"),
+	QString titleText = "KShutdown " KS_FULL_VERSION;
+	QString buildText = "Build: " KS_BUILD;
+	if (Config::isPortable())
+		buildText += " | Portable";
+	auto *titleLabel = new QLabel(
 		"<qt>" \
-		"<h1>KShutdown " + version + "</h1>" +
-		i18n("A graphical shutdown utility") + "<br>" \
-		KS_COPYRIGHT "<br>" \
-		"<a href=\"" KS_HOME_PAGE "\">" KS_HOME_PAGE "</a><br>" \
-		"<pre>" + license + "</pre>" +
+		"<h1 style=\"margin: 0px; white-space: nowrap\">" + titleText + "</h1>" +
+		"<b style=\"white-space: nowrap\">" + buildText + "</b>" \
 		"</qt>"
 	);
+	titleLabel->setAlignment(Qt::AlignLeft | Qt::AlignVCenter);
+
+	QString releaseNotes =
+		"http://kshutdown.sourceforge.net/releases/" KS_FILE_VERSION ".html";
+	auto *aboutLabel = new QLabel(
+		"<qt>" +
+		i18n("A graphical shutdown utility") + "<br />" \
+		"<a href=\"" KS_HOME_PAGE "\">kshutdown.sf.net</a><br />" \
+		"<br />" \
+		"<a href=\"" + releaseNotes + "\">" + i18n("What's New?") + "</a><br />" \
+		"</qt>"
+	);
+	aboutLabel->setAlignment(Qt::AlignLeft | Qt::AlignVCenter);
+	aboutLabel->setOpenExternalLinks(true);
+
+	auto *titleWidget = new QWidget();
+	auto *titleLayout = new QHBoxLayout(titleWidget);
+	titleLayout->setMargin(0_px);
+	titleLayout->setSpacing(20_px);
+	titleLayout->addWidget(iconLabel);
+	titleLayout->addWidget(titleLabel);
+	titleLayout->addStretch();
+
+	auto *aboutTab = new QWidget();
+	auto *aboutLayout = new QVBoxLayout(aboutTab);
+	aboutLayout->setMargin(20_px);
+	aboutLayout->setSpacing(20_px);
+	aboutLayout->addWidget(titleWidget);
+	aboutLayout->addWidget(aboutLabel);
+	aboutLayout->addStretch();
+
+QString licenseText =
+R"(<qt><pre>
+This program is <b>free software</b>; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 2 of the License, or
+(at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but <b>WITHOUT ANY WARRANTY</b>; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+<a href="http://www.gnu.org/licenses/gpl.html">GNU General Public License</a> for more details. (<a href="http://tldrlegal.com/l/gpl2">tl;dr</a>)</pre></qt>)";
+
+	auto *licenseLabel = new QLabel(licenseText);
+	licenseLabel->setAlignment(Qt::AlignLeft | Qt::AlignTop);
+	licenseLabel->setOpenExternalLinks(true);
+
+	auto *aboutQtButton = new U_PUSH_BUTTON(i18n("About Qt"));
+	aboutQtButton->setSizePolicy(QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Preferred));
+	connect(aboutQtButton, SIGNAL(clicked()), U_APP, SLOT(aboutQt()));
+
+	auto *licenseTab = new QWidget();
+	auto *licenseLayout = new QVBoxLayout(licenseTab);
+	licenseLayout->setMargin(20_px);
+	licenseLayout->setSpacing(20_px);
+	licenseLayout->addWidget(new QLabel(KS_COPYRIGHT));
+// TODO: https://sourceforge.net/p/kshutdown/wiki/Credits/
+	licenseLayout->addWidget(licenseLabel);
+	licenseLayout->addStretch();
+	licenseLayout->addWidget(aboutQtButton);
+
+	QPointer<UDialog> dialog = new UDialog(this, i18n("About"), true);
+	#ifdef Q_OS_WIN32
+	dialog->rootLayout()->setMargin(5_px);
+	dialog->rootLayout()->setSpacing(5_px);
+	#endif // Q_OS_WIN32
+
+	auto *tabs = new U_TAB_WIDGET();
+	tabs->addTab(aboutTab, i18n("About"));
+	tabs->addTab(licenseTab, i18n("License"));
+	dialog->mainLayout()->addWidget(tabs);
+
+	dialog->exec();
+	delete dialog;
 }
 #endif // KS_PURE_QT
 
@@ -1128,8 +1318,8 @@ void MainWindow::onActionActivated(int index) {
 		m_actionBox->layout()->addWidget(m_currentActionWidget);
 		m_currentActionWidget->show();
 	}
-	
-	m_actions->setWhatsThis(action->whatsThis());
+
+	m_actions->setToolTip(action->statusTip()/*toolTip()*/);
 
 	onStatusChange(true);
 }
@@ -1189,7 +1379,7 @@ void MainWindow::onFocusChange(QWidget *old, QWidget *now) {
 
 	// update trigger status info on focus gain
 	if (now && (now == m_currentTriggerWidget)) {
-		U_DEBUG << "MainWindow::onFocusChange()" U_END;
+		//U_DEBUG << "MainWindow::onFocusChange()" U_END;
 		onStatusChange(false);
 	}
 }
@@ -1202,8 +1392,12 @@ void MainWindow::onForceClick() {
 		m_force->setChecked(false);
 }
 
+void MainWindow::onMenuHovered(QAction *action) {
+	Utils::showMenuToolTip(action);
+}
+
 void MainWindow::onOKCancel() {
-	U_DEBUG << "MainWindow::onOKCancel()" U_END;
+	//U_DEBUG << "MainWindow::onOKCancel()" U_END;
 	
 	// show error message if selected date/time is invalid
 	if (!m_active) {
@@ -1212,7 +1406,10 @@ void MainWindow::onOKCancel() {
 			dateTimeTrigger &&
 			(dateTimeTrigger->dateTime() <= QDateTime::currentDateTime())
 		) {
-			U_ERROR_MESSAGE(this, i18n("Invalid time: %0").arg(dateTimeTrigger->dateTime().toString(KShutdown::DATE_TIME_DISPLAY_FORMAT)));
+			m_infoWidget->setText(
+				i18n("Invalid time: %0").arg(dateTimeTrigger->dateTime().toString(DateTimeTriggerBase::longDateTimeFormat())),
+				InfoWidget::Type::Error
+			);
 		
 			return;
 		}
@@ -1222,7 +1419,7 @@ void MainWindow::onOKCancel() {
 }
 
 void MainWindow::onPreferences() {
-	U_DEBUG << "MainWindow::onPreferences()" U_END;
+	//U_DEBUG << "MainWindow::onPreferences()" U_END;
 	
 	if (!PasswordDialog::authorizeSettings(this))
 		return;
@@ -1239,27 +1436,33 @@ void MainWindow::onPreferences() {
 	delete dialog;
 }
 
+void MainWindow::onStats() {
+	QPointer<Stats> dialog = new Stats(this);
+	dialog->exec();
+	delete dialog;
+}
+
 void MainWindow::onStatusChange(const bool aUpdateWidgets) {
 	//U_DEBUG << "onStatusChange(" << aUpdateWidgets << ")" U_END;
 
 	Action *action = getSelectedAction();
 	Trigger *trigger = getSelectedTrigger();
 	
-	action->setState(Action::InvalidStatusState);
-	trigger->setState(Trigger::InvalidStatusState);
+	action->setState(Action::State::InvalidStatus);
+	trigger->setState(Trigger::State::InvalidStatus);
 
 	QString displayStatus =
 		m_active
 		? QString::null
-		: getDisplayStatus(DISPLAY_STATUS_HTML | DISPLAY_STATUS_HTML_NO_ACTION);
+		: getDisplayStatus(DISPLAY_STATUS_HTML | DISPLAY_STATUS_HTML_NO_ACTION | DISPLAY_STATUS_OK_HINT);
 	
 	InfoWidget::Type type;
-	if (action->statusType() != InfoWidget::InfoType)
+	if (action->statusType() != InfoWidget::Type::Info)
 		type = action->statusType();
-	else if (trigger->statusType() != InfoWidget::InfoType)
+	else if (trigger->statusType() != InfoWidget::Type::Info)
 		type = trigger->statusType();
 	else
-		type = InfoWidget::InfoType;
+		type = InfoWidget::Type::Info;
 	
 	if (aUpdateWidgets) {
 		updateWidgets();
@@ -1289,7 +1492,7 @@ void MainWindow::onTriggerActivated(int index) {
 		m_currentTriggerWidget->show();
 	}
 	
-	m_triggers->setWhatsThis(trigger->whatsThis());
+	m_triggers->setToolTip(trigger->toolTip());
 
 	onStatusChange(true);
 }
