@@ -25,9 +25,12 @@
 
 #ifdef KS_PURE_QT
 	class QComboBox;
+#elif defined(KS_KF5)
+	class QComboBox;
+	
+	class KActionCollection;
 #else
 	class KActionCollection;
-	class KCmdLineArgs;
 	class KComboBox;
 #endif // KS_PURE_QT
 
@@ -44,6 +47,7 @@ class USystemTray;
 using namespace KShutdown;
 
 class MainWindow: public U_MAIN_WINDOW {
+	friend class Mod;
 	friend class TimeOption;
 	Q_OBJECT
 	#ifdef KS_PURE_QT
@@ -51,27 +55,32 @@ class MainWindow: public U_MAIN_WINDOW {
 	Q_CLASSINFO("D-Bus Interface", "net.sf.kshutdown.MainWindow")
 	#endif // KS_PURE_QT
 public:
-	enum DisplayStatus {
+	enum/* non-class */DisplayStatus {
 		DISPLAY_STATUS_HTML = 1 << 0,
 		DISPLAY_STATUS_HTML_NO_ACTION = 1 << 1,
 		DISPLAY_STATUS_SIMPLE = 1 << 2,
-		DISPLAY_STATUS_APP_NAME = 1 << 3
+		DISPLAY_STATUS_APP_NAME = 1 << 3,
+		DISPLAY_STATUS_OK_HINT = 1 << 4
 	};
 	virtual ~MainWindow();
 	QHash<QString, Action*> actionHash() const { return m_actionHash; }
+	QList<Action*> actionList() const { return m_actionList; }
+	U_ACTION *cancelAction() const { return m_cancelAction; }
 	static bool checkCommandLine();
 	QString getDisplayStatus(const int options);
 	Action *getSelectedAction() const;
 	Trigger *getSelectedTrigger() const;
+	inline InfoWidget *infoWidget() { return m_infoWidget; }
 	static void init();
+	inline U_PUSH_BUTTON *okCancelButton() { return m_okCancelButton; }
 	inline ProgressBar *progressBar() { return m_progressBar; }
-	inline static MainWindow *self() {
+	static MainWindow *self() {
 		if (!m_instance)
 			m_instance = new MainWindow();
 
 		return m_instance;
 	}
-	void maybeShow();
+	bool maybeShow();
 	void setTime(const QString &trigger, const QTime &time, const bool absolute);
 	QHash<QString, Trigger*> triggerHash() const { return m_triggerHash; }
 public slots:
@@ -87,7 +96,7 @@ public slots:
 	Q_SCRIPTABLE void setWaitForProcess(const qint64 pid);
 	void writeConfig();
 protected:
-	virtual void closeEvent(QCloseEvent *e);
+	virtual void closeEvent(QCloseEvent *e) override;
 private:
 	Q_DISABLE_COPY(MainWindow)
 #ifdef KS_NATIVE_KDE
@@ -119,13 +128,15 @@ private:
 	U_COMBO_BOX *m_triggers;
 	U_PUSH_BUTTON *m_okCancelButton;
 	USystemTray *m_systemTray;
-	MainWindow();
+	explicit MainWindow();
 	static void addAction(Action *action);
 	static void addTrigger(Trigger *trigger);
+	U_ACTION *createQuitAction();
+	void initFileMenu(U_MENU *fileMenu, const bool addQuitAction);
 	void initMenuBar();
 	void initTriggers();
 	void initWidgets();
-	void pluginConfig(const bool read);
+	static void pluginConfig(const bool read);
 	void readConfig();
 	void setTitle(const QString &plain, const QString &html);
 	void updateWidgets();
@@ -142,9 +153,11 @@ private slots:
 #endif // KS_NATIVE_KDE
 	void onFocusChange(QWidget *old, QWidget *now);
 	void onForceClick();
+	void onMenuHovered(QAction *action);
 	void onOKCancel();
 	void onPreferences();
 	void onQuit();
+	void onStats();
 	void onStatusChange(const bool updateWidgets);
 	void onTriggerActivated(int index);
 };
