@@ -15,10 +15,13 @@
 // with this program; if not, write to the Free Software Foundation, Inc.,
 // 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
+#include "password.h"
+
 #include "config.h"
 #include "infowidget.h"
+#include "log.h"
 #include "mainwindow.h"
-#include "password.h"
+#include "utils.h"
 
 #ifdef KS_NATIVE_KDE
 	#include <KPasswordDialog>
@@ -26,13 +29,13 @@
 
 #ifdef KS_PURE_QT
 	#include <QInputDialog>
-	#include <QPointer>
 #endif // KS_PURE_QT
 
 #include <QCheckBox>
 #include <QCryptographicHash>
 #include <QFormLayout>
 #include <QLabel>
+#include <QPointer>
 
 // PasswordDialog
 
@@ -40,15 +43,9 @@
 
 PasswordDialog::PasswordDialog(QWidget *parent) :
 	UDialog(parent, i18n("Enter New Password"), false) {
-	U_DEBUG << "PasswordDialog::PasswordDialog()" U_END;
-
-	QVBoxLayout *mainLayout = this->mainLayout();
-
-	// form
-
-	QFormLayout *formLayout = new QFormLayout();
-	mainLayout->addLayout(formLayout);
+	//U_DEBUG << "PasswordDialog::PasswordDialog()" U_END;
 
+// TODO: show warning if Caps Lock is turned on (no reliable solution in Qt)
 	m_password = new U_LINE_EDIT();
 	#if QT_VERSION >= 0x050200
 	m_password->setClearButtonEnabled(true);
@@ -58,8 +55,7 @@ PasswordDialog::PasswordDialog(QWidget *parent) :
 		m_password, SIGNAL(textChanged(const QString &)),
 		SLOT(onPasswordChange(const QString &))
 	);
-	formLayout->addRow(i18n("Password:"), m_password);
-	
+
 	m_confirmPassword = new U_LINE_EDIT();
 	#if QT_VERSION >= 0x050200
 	m_confirmPassword->setClearButtonEnabled(true);
@@ -69,36 +65,39 @@ PasswordDialog::PasswordDialog(QWidget *parent) :
 		m_confirmPassword, SIGNAL(textChanged(const QString &)),
 		SLOT(onConfirmPasswordChange(const QString &))
 	);
-	formLayout->addRow(i18n("Confirm Password:"), m_confirmPassword);
-		
-	// status/hint
-		
+
 	m_status = new InfoWidget(this);
-	m_status->setText(
-		"<qt>" +
-		i18n("The password will be saved as SHA-1 hash.") + "<br>" +
-		i18n("Short password can be easily cracked.") +
-		"</qt>",
-		InfoWidget::WarningType
-	);
-	mainLayout->addSpacing(10);
-	mainLayout->addWidget(m_status);
-	
-	addButtonBox();
+
+	auto *layout = new QFormLayout();
+	// HACK: force "correct" alignment
+	layout->setLabelAlignment(Qt::AlignRight);
+	layout->addRow(i18n("Password:"), m_password);
+	layout->addRow(i18n("Confirm Password:"), m_confirmPassword);
+
+	mainLayout()->addLayout(layout);
+	mainLayout()->addWidget(m_status);
 
 	m_password->setFocus();
-	
+
 	updateStatus();
 }
 
 PasswordDialog::~PasswordDialog() {
-	U_DEBUG << "PasswordDialog::~PasswordDialog()" U_END;
+	//U_DEBUG << "PasswordDialog::~PasswordDialog()" U_END;
+	m_password->setText("");
+	m_confirmPassword->setText("");
 }
 
 void PasswordDialog::apply() {
+	Log::warning("Password changed/set by user");
+
 	Config *config = Config::user();
 	config->beginGroup("Password Protection");
-	config->write("Hash", toHash(m_password->text()));
+
+	QString password = m_password->text();
+	config->write("Hash", toHash(password));
+	clearPassword(password);
+
 	config->endGroup();
 	config->sync();
 }
@@ -115,27 +114,27 @@ bool PasswordDialog::authorize(QWidget *parent, const QString &caption, const QS
 	if (hash.isEmpty())
 		return true;
 
-	QString password = QString::null;
 	QString prompt = i18n("Enter password to perform action: %0").arg(caption);
 
+retry:
+
 	#ifdef KS_NATIVE_KDE
 	QPointer<KPasswordDialog> dialog = new KPasswordDialog(parent);
-	dialog->setPixmap(U_ICON("kshutdown").pixmap(48, 48));
+	dialog->setPixmap(U_ICON("kshutdown").pixmap(48_px, 48_px));
 	dialog->setPrompt(prompt);
 	bool ok = dialog->exec();
-	if (ok)
-		password = dialog->password();
+	QString password = ok ? dialog->password() : QString::null;
 	delete dialog;
 	
 	if (!ok)
 		return false;
 	#else
 	bool ok;
-	password = QInputDialog::getText( // krazy:exclude=qclasses
+	QString password = QInputDialog::getText( // krazy:exclude=qclasses
 		parent,
 		"KShutdown", // title
 		prompt,
-		QLineEdit::Password, // krazy:exclude=qclasses
+		U_LINE_EDIT::Password,
 		"",
 		&ok
 	);
@@ -144,12 +143,19 @@ bool PasswordDialog::authorize(QWidget *parent, const QString &caption, const QS
 		return false;
 	#endif // KS_NATIVE_KDE
 
-	if (hash != toHash(password)) {
+	QString enteredHash = toHash(password);
+	clearPassword(password);
+
+	if (hash != enteredHash) {
+		Log::warning("Invalid password for action: " + userAction);
+
 		U_ERROR_MESSAGE(parent, i18n("Invalid password"));
-		
-		return false;
+
+		goto retry; // goto considered useful
 	}
-	
+
+	Log::warning("Action successfully authenticated (using password): " + userAction);
+
 	return true;
 }
 
@@ -157,40 +163,52 @@ bool PasswordDialog::authorizeSettings(QWidget *parent) {
 	return PasswordDialog::authorize(parent, i18n("Preferences"), "action/settings");
 }
 
+void PasswordDialog::clearPassword(QString &password) {
+	password.fill('\0');
+	password.clear();
+}
+
 QString PasswordDialog::toHash(const QString &password) {
 	if (password.isEmpty())
 		return "";
-	
-	QByteArray hash = QCryptographicHash::hash(("kshutdown-" + password).toUtf8(), QCryptographicHash::Sha1);
-	
+
+// TODO: consider other algorithms introduced in Qt 5.x <http://doc.qt.io/qt-5/qcryptographichash.html#Algorithm-enum>
+	QString saltedString = "kshutdown-" + password;
+	QByteArray saltedArray = saltedString.toUtf8();
+
+	QByteArray hash = QCryptographicHash::hash(saltedArray, QCryptographicHash::Sha1);
+
+	clearPassword(saltedString);
+	saltedArray.fill('\0');
+	saltedArray.clear();
+
 	return QString(hash.toHex());
 }
 
 // private:
 
 void PasswordDialog::updateStatus() {
-	int minLength = 6;
-	bool ok = m_password->text().length() >= minLength;
+	QString password = m_password->text();
+	QString confirmPassword = m_confirmPassword->text();
+
+	int minLength = 12;
+	bool ok = password.length() >= minLength;
 	if (!ok) {
-		m_status->setText(i18n("Password is too short (need %0 characters or more)").arg(minLength), InfoWidget::ErrorType);
+		m_status->setText(i18n("Password is too short (need %0 characters or more)").arg(minLength), InfoWidget::Type::Error);
 	}
 	else {
-		ok = (m_password->text() == m_confirmPassword->text());
-		if (!ok) {
-			m_status->setText(i18n("Confirmation password is different"), InfoWidget::ErrorType);
-		}
-		else {
-			if (m_password->text() == "123456")
-				m_status->setText(":-(", InfoWidget::WarningType);
-			else if (m_password->text() == "dupa.8")
-				m_status->setText("O_o", InfoWidget::InfoType);
-			else
-				m_status->setText(QString::null, InfoWidget::ErrorType);
-		}
+		ok = (password == confirmPassword);
+		if (!ok)
+			m_status->setText(i18n("Confirmation password is different"), InfoWidget::Type::Error);
+		else
+			m_status->setText(QString::null, InfoWidget::Type::Error);
 	}
 
 	acceptButton()->setEnabled(ok);
 	resize(sizeHint());
+
+	clearPassword(password);
+	clearPassword(confirmPassword);
 }
 
 // private slots:
@@ -215,11 +233,11 @@ PasswordPreferences::PasswordPreferences(QWidget *parent) :
 	QWidget(parent),
 	m_configKeyRole(Qt::UserRole)
 {
-	U_DEBUG << "PasswordPreferences::PasswordPreferences()" U_END;
+	//U_DEBUG << "PasswordPreferences::PasswordPreferences()" U_END;
 
 	QVBoxLayout *mainLayout = new QVBoxLayout(this);
-	mainLayout->setMargin(10);
-	mainLayout->setSpacing(10);
+	mainLayout->setMargin(10_px);
+	mainLayout->setSpacing(10_px);
 
 	m_enablePassword = new QCheckBox(i18n("Enable Password Protection"));
 	Config *config = Config::user();
@@ -232,8 +250,8 @@ PasswordPreferences::PasswordPreferences(QWidget *parent) :
 	config->endGroup();
 	
 	connect(
-		m_enablePassword, SIGNAL(stateChanged(int)),
-		SLOT(onEnablePassword(int))
+		m_enablePassword, SIGNAL(clicked(bool)),
+		SLOT(onEnablePassword(bool))
 	);
 	mainLayout->addWidget(m_enablePassword);
 	
@@ -245,7 +263,7 @@ PasswordPreferences::PasswordPreferences(QWidget *parent) :
 	
 	addItem("action/settings", i18n("Settings (recommended)"), U_ICON("configure"));
 	
-	foreach (const Action *action, MainWindow::self()->actionHash().values()) {
+	foreach (const Action *action, MainWindow::self()->actionList()) {
 		addItem(
 			"kshutdown/action/" + action->id(),
 			action->originalText(),
@@ -254,29 +272,32 @@ PasswordPreferences::PasswordPreferences(QWidget *parent) :
 	}
 
 	addItem("kshutdown/action/cancel", i18n("Cancel"), U_ICON("dialog-cancel"));
-	addItem("action/file_quit", i18n("Quit"), U_ICON("application-exit"));
+	addItem("action/file_quit", i18n("Quit KShutdown"), U_ICON("application-exit"));
 
 	userActionListLabel->setBuddy(m_userActionList);
 	mainLayout->addWidget(m_userActionList);
 
 	InfoWidget *kioskInfo = new InfoWidget(this);
-	kioskInfo->setText("<qt>" + i18n("See Also: %0").arg("<a href=\"http://sourceforge.net/p/kshutdown/wiki/Kiosk/\">Kiosk</a>") + "</qt>", InfoWidget::InfoType);
+	kioskInfo->setText("<qt>" + i18n("See Also: %0").arg("<a href=\"http://sourceforge.net/p/kshutdown/wiki/Kiosk/\">Kiosk</a>") + "</qt>", InfoWidget::Type::Info);
 	mainLayout->addWidget(kioskInfo);
 	
-	updateWidgets(m_enablePassword->checkState() == Qt::Checked);
+	updateWidgets(m_enablePassword->isChecked());
 }
 
 PasswordPreferences::~PasswordPreferences() {
-	U_DEBUG << "PasswordPreferences::~PasswordPreferences()" U_END;
+	//U_DEBUG << "PasswordPreferences::~PasswordPreferences()" U_END;
 }
 
 void PasswordPreferences::apply() {
-	U_DEBUG << "PasswordPreferences::apply()" U_END;
+	//U_DEBUG << "PasswordPreferences::apply()" U_END;
 	
 	Config *config = Config::user();
 	config->beginGroup("Password Protection");
-	if (m_enablePassword->checkState() != Qt::Checked)
+	if (!m_enablePassword->isChecked()) {
+		Log::warning("Password protection disabled by user");
+
 		config->write("Hash", "");
+	}
 
 	int count = m_userActionList->count();
 	for (int i = 0; i < count; i++) {
@@ -310,14 +331,16 @@ void PasswordPreferences::updateWidgets(const bool passwordEnabled) {
 
 // private slots:
 
-void PasswordPreferences::onEnablePassword(int state) {
-	if (state == Qt::Checked) {
+void PasswordPreferences::onEnablePassword(bool checked) {
+	//U_DEBUG << "PasswordPreferences::onEnablePassword: " << checked U_END;
+	
+	if (checked) {
 		QPointer<PasswordDialog> dialog = new PasswordDialog(this);
 		if (dialog->exec() == PasswordDialog::Accepted)
 			dialog->apply();
 		else
-			m_enablePassword->setCheckState(Qt::Unchecked);
+			m_enablePassword->setChecked(false);
 		delete dialog;
 	}
-	updateWidgets(m_enablePassword->checkState() == Qt::Checked);
+	updateWidgets(m_enablePassword->isChecked());
 }
