@@ -32,6 +32,7 @@ public:
 	void apply();
 	static bool authorize(QWidget *parent, const QString &caption, const QString &userAction);
 	static bool authorizeSettings(QWidget *parent);
+	static void clearPassword(QString &password);
 	static QString toHash(const QString &password);
 private:
 	Q_DISABLE_COPY(PasswordDialog)
@@ -58,7 +59,7 @@ private:
 	QListWidgetItem *addItem(const QString &key, const QString &text, const QIcon &icon);
 	void updateWidgets(const bool passwordEnabled);
 private slots:
-	void onEnablePassword(int state);
+	void onEnablePassword(bool checked);
 };
 
 #endif // KSHUTDOWN_PASSWORD_H
