@@ -16,6 +16,8 @@
 // 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
 #include "config.h"
+#include "mainwindow.h"
+#include "mod.h"
 #include "password.h"
 #include "progressbar.h"
 #include "pureqt.h"
@@ -24,8 +26,9 @@
 #include <QDesktopWidget>
 #include <QMouseEvent>
 #include <QPainter>
+#include <QTimer>
 
-#ifdef KS_NATIVE_KDE
+#if defined(KS_NATIVE_KDE) && !defined(KS_KF5)
 	#include <KColorDialog>
 #else
 	#include <QColorDialog>
@@ -34,7 +37,7 @@
 // public
 
 ProgressBar::~ProgressBar() {
-	U_DEBUG << "ProgressBar::~ProgressBar()" U_END;
+	//U_DEBUG << "ProgressBar::~ProgressBar()" U_END;
 }
 
 void ProgressBar::setAlignment(const Qt::Alignment value, const bool updateConfig) {
@@ -46,24 +49,39 @@ void ProgressBar::setAlignment(const Qt::Alignment value, const bool updateConfi
 
 	m_alignment = value;
 	QDesktopWidget *desktop = QApplication::desktop();
-	resize(desktop->width() - 4, height());
+
+	int margin = 2_px;
+
+	resize(desktop->width() - margin * 2, height());
 	if (m_alignment.testFlag(Qt::AlignBottom)) {
-		move(2, desktop->height() - height());
+		move(margin, desktop->height() - height());
 	}
 	// Qt::AlignTop
 	else {
-		move(2, 0);
+		move(margin, 0_px);
+	}
+}
+
+
+void ProgressBar::setDemo(const bool active) {
+	U_DEBUG << "ProgressBar::setDemo: " << active U_END;
+
+	if (active) {
+		m_demoWidth = 0_px;
+		m_demoTimer->start(50);
+	}
+	else {
+		m_demoTimer->stop();
 	}
 }
 
 void ProgressBar::setHeight(const int value) {
-	int newHeight = (value < 2) ? 2 : value;
-	resize(width(), newHeight);
+	resize(width(), qMax(2_px, value));
 }
 
 void ProgressBar::setTotal(const int total) {
 	m_total = total;
-	m_completeWidth = 0; // reset
+	m_completeWidth = 0_px; // reset
 }
 
 void ProgressBar::setValue(const int value) {
@@ -81,69 +99,84 @@ void ProgressBar::setValue(const int value) {
 
 // protected
 
-void ProgressBar::mousePressEvent(QMouseEvent *e) {
-	if (Utils::isRestricted("kshutdown/progress_bar/menu"))
+void ProgressBar::contextMenuEvent(QContextMenuEvent *e) {
+	if (
+		Utils::isArg("hide-ui") ||
+		Utils::isRestricted("kshutdown/progress_bar/menu")
+	)
 		return;
 
-	if (e->button() == Qt::RightButton) {
-		// show popup menu
-		U_MENU *menu = new U_MENU(this);
+	// show popup menu
+	U_MENU *menu = new U_MENU(this);
 
-		#ifdef KS_NATIVE_KDE
-		menu->addTitle(U_APP->windowIcon(), KGlobal::caption());
-		#endif // KS_NATIVE_KDE
-
-		bool canConfigure = !Utils::isRestricted("action/options_configure");
-
-		menu->addAction(i18n("Hide"), this, SLOT(onHide()));
-		menu->addAction(i18n("Set Color..."), this, SLOT(onSetColor()))
-			->setEnabled(canConfigure);
-
-		#ifdef KS_NATIVE_KDE
-		menu->addTitle(i18n("Position"));
+	Utils::addTitle(menu, U_APP->windowIcon(),
+		i18n("Progress Bar") + " - " +
+		#ifdef KS_KF5
+		QApplication::applicationDisplayName()
+		#elif defined(KS_NATIVE_KDE)
+		KGlobal::caption()
 		#else
-			#if QT_VERSION >= 0x050200
-			menu->addSection(i18n("Position"));
-			#else
-			menu->addSeparator();
-			#endif // QT_VERSION
-		#endif // KS_NATIVE_KDE
+		i18n("KShutdown")
+		#endif // KS_KF5
+	);
 
-		QActionGroup *positionGroup = new QActionGroup(this);
+	bool canConfigure = !Utils::isRestricted("action/options_configure");
 
-		QAction *a = menu->addAction(i18n("Top"), this, SLOT(onSetTopAlignment()));
-		makeRadioButton(a, positionGroup, m_alignment.testFlag(Qt::AlignTop));
+	menu->addAction(i18n("Hide"), this, SLOT(onHide()));
+	menu->addAction(i18n("Set Color..."), this, SLOT(onSetColor()))
+		->setEnabled(canConfigure);
+
+	// position
+
+	U_MENU *positionMenu = new U_MENU(i18n("Position"), menu);
+
+	QActionGroup *positionGroup = new QActionGroup(this);
+
+	QAction *a = positionMenu->addAction(i18n("Top"), this, SLOT(onSetTopAlignment()));
+	makeRadioButton(a, positionGroup, m_alignment.testFlag(Qt::AlignTop));
 		
-		a = menu->addAction(i18n("Bottom"), this, SLOT(onSetBottomAlignment()));
-		makeRadioButton(a, positionGroup, m_alignment.testFlag(Qt::AlignBottom));
+	a = positionMenu->addAction(i18n("Bottom"), this, SLOT(onSetBottomAlignment()));
+	makeRadioButton(a, positionGroup, m_alignment.testFlag(Qt::AlignBottom));
 
-		#ifdef KS_NATIVE_KDE
-		menu->addTitle(i18n("Size"));
-		#else
-			#if QT_VERSION >= 0x050200
-			menu->addSection(i18n("Size"));
-			#else
-			menu->addSeparator();
-			#endif // QT_VERSION
-		#endif // KS_NATIVE_KDE
+	U_MENU *sizeMenu = new U_MENU(i18n("Size"), menu);
 
-		QActionGroup *sizeGroup = new QActionGroup(this);
+	// size
+
+	QActionGroup *sizeGroup = new QActionGroup(this);
 		
-		a = menu->addAction(i18n("Small"), this, SLOT(onSetSizeSmall()));
-		makeRadioButton(a, sizeGroup, height() == SmallSize);
+	a = sizeMenu->addAction(i18n("Small"), this, SLOT(onSetSizeSmall()));
+	makeRadioButton(a, sizeGroup, height() == SmallSize);
 
-		a = menu->addAction(i18n("Normal"), this, SLOT(onSetSizeNormal()));
-		makeRadioButton(a, sizeGroup, height() == NormalSize);
+	a = sizeMenu->addAction(i18n("Normal"), this, SLOT(onSetSizeNormal()));
+	makeRadioButton(a, sizeGroup, height() == NormalSize);
 
-		a = menu->addAction(i18n("Medium"), this, SLOT(onSetSizeMedium()));
-		makeRadioButton(a, sizeGroup, height() == MediumSize);
+	a = sizeMenu->addAction(i18n("Medium"), this, SLOT(onSetSizeMedium()));
+	makeRadioButton(a, sizeGroup, height() == MediumSize);
 
-		a = menu->addAction(i18n("Large"), this, SLOT(onSetSizeLarge()));
-		makeRadioButton(a, sizeGroup, height() == LargeSize);
+	a = sizeMenu->addAction(i18n("Large"), this, SLOT(onSetSizeLarge()));
+	makeRadioButton(a, sizeGroup, height() == LargeSize);
+
+	menu->addSeparator();
+	menu->addMenu(positionMenu);
+	menu->addMenu(sizeMenu);
+	menu->addSeparator();
+	menu->addAction(MainWindow::self()->cancelAction());
+
+	menu->popup(e->globalPos());
+	e->accept();
+}
+
+void ProgressBar::mousePressEvent(QMouseEvent *e) {
+	if (!Utils::isArg("hide-ui") && (e->button() == Qt::LeftButton)) {
+		MainWindow *mainWindow = MainWindow::self();
+		mainWindow->show();
+		mainWindow->activateWindow();
 
-		menu->popup(e->globalPos());
 		e->accept();
+
+		return;
 	}
+
 	QWidget::mousePressEvent(e);
 }
 
@@ -151,14 +184,24 @@ void ProgressBar::paintEvent(QPaintEvent *e) {
 	Q_UNUSED(e)
 
 	QPainter g(this);
+
+	int x = 0_px;
+	int y = 0_px;
 	int w = width();
 	int h = height();
-	g.fillRect(0, 0, w, h, palette().window());
 
-	if ((m_completeWidth <= 0) || (m_total <= 0) || (m_value <= 0))
-		return;
+	if (m_demoTimer->isActive()) {
+		g.fillRect(x, y, w, h, Qt::black);
+		g.fillRect(x, y, qMin(m_demoWidth, w), h, m_demoColor);
+	}
+	else {
+		g.fillRect(x, y, w, h, palette().window());
 
-	g.fillRect(0, 0, m_completeWidth, h, palette().windowText());
+		if ((m_completeWidth <= 0_px) || (m_total <= 0) || (m_value <= 0))
+			return;
+
+		g.fillRect(x, y, m_completeWidth, h, palette().windowText());
+	}
 }
 
 // private
@@ -174,25 +217,36 @@ ProgressBar::ProgressBar() // public
 		Qt::WindowStaysOnTopHint |
 		Qt::X11BypassWindowManagerHint |
 		Qt::Tool
-	),
-	m_completeWidth(0),
-	m_total(0),
-	m_value(0) {
+	) {
 	
-	U_DEBUG << "ProgressBar::ProgressBar()" U_END;
+	//U_DEBUG << "ProgressBar::ProgressBar()" U_END;
+
+	m_demoTimer = new QTimer(this);
+	connect(m_demoTimer, SIGNAL(timeout()), SLOT(onDemoTimeout()));
 
 	setAttribute(Qt::WA_AlwaysShowToolTips, true);
 	setObjectName("progress-bar");
-	setWindowTitle("KShutdown - " + i18n("Progress Bar"));
+
+	setWindowTitle(i18n("Progress Bar") + " - " + "KShutdown");
+	setToolTip(windowTitle());
+
+	QVariant opacityVariant = Mod::get("ui-progress-bar-opacity", 1.0f);
+	bool opacityOK = false;
+	qreal opacity = opacityVariant.toReal(&opacityOK);
+	if (opacityOK)
+		setWindowOpacity(opacity);
 
 	QPalette p;
-	QColor background = QColor(Qt::black);
+
+	QColor background = Mod::getColor("ui-progress-bar-window-color", Qt::black);
 	p.setColor(QPalette::Window, background);
 
-// TODO: transparent
 	Config *config = Config::user();
 	config->beginGroup("Progress Bar");
-	QColor defaultForeground = QColor(0xF8FFBF /* lime 1 */);
+
+	QColor defaultForeground = 0xF8FFBF /* lime 1 */;
+	m_demoColor = defaultForeground;
+
 	QColor foreground = config->read("Foreground Color", defaultForeground).value<QColor>();
 	
 	setHeight(qBound(SmallSize, (Size)config->read("Size", NormalSize).toInt(), LargeSize));
@@ -232,6 +286,24 @@ void ProgressBar::setSize(const Size size) {
 
 // private slots
 
+void ProgressBar::onDemoTimeout() {
+	m_demoWidth += 5_px;
+	if (m_demoWidth > width() / 3) {
+		m_demoWidth = 0_px;
+		m_demoTimer->stop();
+		repaint();
+
+		return;
+	}
+
+	int h, s, v;
+	m_demoColor.getHsv(&h, &s, &v);
+	h = (h + 5) % 360;
+	m_demoColor.setHsv(h, s, v);
+
+	repaint();
+}
+
 void ProgressBar::onHide() {
 	if (!authorize())
 		return;
@@ -258,12 +330,12 @@ void ProgressBar::onSetColor() {
 		return;
 
 	QColor currentColor = palette().color(QPalette::WindowText);
-	#ifdef KS_NATIVE_KDE
+	#if defined(KS_NATIVE_KDE) && !defined(KS_KF5)
 	QColor newColor;
 	if (KColorDialog::getColor(newColor, currentColor, this) != KColorDialog::Accepted)
 		return;
 	#else
-	QColor newColor = QColorDialog::getColor( // krazy:exclude=qclasses
+	QColor newColor = QColorDialog::getColor(
 		currentColor,
 		this,
 		QString::null // use default title
