@@ -29,7 +29,6 @@
 	#ifndef WIN32_LEAN_AND_MEAN
 		#define WIN32_LEAN_AND_MEAN
 	#endif // WIN32_LEAN_AND_MEAN
-	#define _WIN32_WINNT 0x0500 // for LockWorkStation, etc
 	#include <windows.h>
 
 	#define KS_TRIGGER_PROCESS_MONITOR
@@ -40,11 +39,10 @@
 	#include <sys/types.h>
 #endif // KS_TRIGGER_PROCESS_MONITOR_UNIX
 
-#include <QProcess>
-
-class Process: public QObject {
+class Process final: public QObject {
 public:
 	explicit Process(QObject *parent, const QString &command);
+	U_ICON icon() const;
 	bool isRunning() const;
 	#ifdef KS_TRIGGER_PROCESS_MONITOR_UNIX
 	inline bool own() const { return m_own; }
@@ -56,11 +54,12 @@ public:
 	inline HWND windowHandle() const { return m_windowHandle; }
 	inline void setWindowHandle(const HWND windowHandle) { m_windowHandle = windowHandle; }
 	#endif // KS_TRIGGER_PROCESS_MONITOR_WIN
-	QString toString() const;
+	inline QString toString() const { return m_stringCache; }
 private:
 	Q_DISABLE_COPY(Process)
 	friend class ProcessMonitor;
 	QString m_command; // a process command or window title (e.g. "firefox")
+	QString m_stringCache = "<THEDAILYWTF>";
 	
 	#ifdef KS_TRIGGER_PROCESS_MONITOR_UNIX
 	bool m_own;
@@ -73,33 +72,34 @@ private:
 	bool m_visible;
 	HWND m_windowHandle;
 	#endif // KS_TRIGGER_PROCESS_MONITOR_WIN
+
+	void makeStringCache();
 };
 
 class ProcessMonitor: public KShutdown::Trigger {
 	Q_OBJECT
 public:
-	ProcessMonitor();
+	explicit ProcessMonitor();
 	void addProcess(Process *process);
-	virtual bool canActivateAction();
-	virtual QWidget *getWidget();
+	virtual bool canActivateAction() override;
+	virtual QWidget *getWidget() override;
+	virtual void readConfig(Config *config) override;
+	virtual void writeConfig(Config *config) override;
 	void setPID(const qint64 pid);
 private:
 	Q_DISABLE_COPY(ProcessMonitor)
 	QList<Process*> m_processList;
-	QProcess *m_refreshProcess;
-	QString m_refreshBuf;
-	QWidget *m_widget;
-	U_COMBO_BOX *m_processesComboBox;
+	QString m_recentCommand = "";
+	QWidget *m_widget = nullptr;
+	U_COMBO_BOX *m_processesComboBox = nullptr;
 	void clearAll();
 	void errorMessage(const QString &message);
+	void refreshProcessList();
 	void updateStatus(const Process *process);
 public slots:
 	void onRefresh();
 private slots:
-	void onError(QProcess::ProcessError error);
-	void onFinished(int exitCode, QProcess::ExitStatus exitStatus);
 	void onProcessSelect(const int index);
-	void onReadyReadStandardOutput();
 };
 
 #endif // KSHUTDOWN_PROCESSMONITOR_H
