@@ -18,15 +18,29 @@
 #ifndef KSHUTDOWN_UTILS_H
 #define KSHUTDOWN_UTILS_H
 
+#include "pureqt.h"
+
+#include <QProcess>
 #include <QProcessEnvironment>
 #include <QString>
 
+#ifdef KS_NATIVE_KDE
+	#ifdef KS_KF5
+		#include <QCommandLineParser>
+	#else
+		#include <KCmdLineArgs>
+	#endif // KS_KF5
+#endif // KS_NATIVE_KDE
+
+// TODO: use std::chrono and "hrs" literals #C++14 #experimental
+inline int operator "" _px(unsigned long long int value) { return value; }
+
+class QAction;
 class QWidget;
 
-class KCmdLineArgs;
-
-class Utils {
+class Utils final {
 public:
+	static void addTitle(U_MENU *menu, const QIcon &icon, const QString &text);
 	static QString getOption(const QString &name);
 	static QString getTimeOption();
 	static QString getUser();
@@ -35,31 +49,46 @@ public:
 	static bool isArg(const QString &name);
 	static bool isCinnamon();
 	static bool isEnlightenment();
+	static bool isGUI();
 	static bool isHelpArg();
 	static bool isGNOME();
 	static bool isGTKStyle();
 	static bool isHaiku();
 	static bool isKDEFullSession();
-	static bool isKDE_4();
+	static bool isKDE();
 	static bool isLXDE();
+	static bool isLXQt();
 	static bool isMATE();
+	static bool isOpenbox();
 	static bool isRazor();
 	static bool isRestricted(const QString &action);
+	static bool isTrinity();
 	static bool isUnity();
 	static bool isXfce();
+	#ifdef KS_KF5
+	static inline QCommandLineParser *parser() { return m_args; }
+	static inline void setParser(QCommandLineParser *value) { m_args = value; }
+	#endif // KS_KF5
+	static QString read(QProcess &process, bool &ok);
 	static void setFont(QWidget *widget, const int relativeSize, const bool bold);
+	static void showMenuToolTip(QAction *action);
 	static void shutDown();
 	static QString trim(QString &text, const int maxLength);
 private:
+	static bool m_gui;
 #ifdef KS_NATIVE_KDE
+	#ifdef KS_KF5
+	static QCommandLineParser *m_args;
+	#else
 	static KCmdLineArgs *m_args;
+	#endif // KS_KF5
 #else
 	static QStringList m_args;
 #endif // KS_NATIVE_KDE
 	static QProcessEnvironment m_env;
 	static QString m_desktopSession;
 	static QString m_xdgCurrentDesktop;
-	Utils() { }
+	explicit Utils() { }
 };
 
 #endif // KSHUTDOWN_UTILS_H
