@@ -1,5 +1,6 @@
 #!/bin/bash
 
+# SEE: https://github.com/Krazy-collection/krazy
 # DOC: http://techbase.kde.org/Development/Tutorials/Code_Checking
 
 if [ ! -f "ChangeLog" ]; then
@@ -16,5 +17,5 @@ if [ -z "$dir" ]; then
 fi
 
 pushd "$dir"
-krazy2all
-popd
\ No newline at end of file
+krazy2all --check-sets c++,foss,kde5,qt5
+popd
